@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2012 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This file was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -30,7 +30,7 @@
 
 #define MASK_4K 	0xfff
 
-/* See http://download.intel.com/technology/computing/vptech/Intel(r)_VT_for_Direct_IO.pdf */
+/* See https://software.intel.com/content/dam/develop/external/us/en/documents-tps/vt-directed-io-spec.pdf */
 
 /* DMA Remapping Reporting Table (DMAR) */
 struct acpi_table_dmar {
@@ -71,7 +71,7 @@ struct acpi_table_rmrr {
 } __attribute__ ((packed));
 
 enum acpi_dev_scope_type {
-	ACPI_DEV_ENDPOINT=0x01,
+	ACPI_DEV_ENDPOINT = 0x01,
 	ACPI_DEV_P2PBRIDGE,
 	ACPI_DEV_IOAPIC,
 	ACPI_DEV_HPET,
@@ -109,13 +109,13 @@ static int read_pci_device_secondary_bus_number(const uint8_t seg,
 	size_t count;
 
 	snprintf(path, sizeof(path),
-		"/sys/bus/pci/devices/%04x:%02x:%02x.%d/config",
+		"/sys/bus/pci/devices/%04x:%02x:%02x.%" PRIu8 "/config",
 		seg, bus, dev, fn);
 	if ((file = fopen(path, "r")) == NULL)
 		return -1;
 
 	count = fread(configs, sizeof(char), 64, file);
-	fclose(file);
+	(void)fclose(file);
 
 	if (count < 64)
 		return -1;
@@ -166,8 +166,11 @@ static int acpi_parse_one_dev_scope(fwts_framework *fw,
 			goto error;
 		dev_type = read_pci_device_secondary_bus_number(seg, bus,
 			path->dev, path->fn, &sec_bus);
-		if (dev_type < 0) /* no such device */
+		if (dev_type < 0) {	/* no such device */
+			fwts_warning(fw, "PCI device %04Xh:%02Xh:%02Xh.%02Xh is not found.",
+				     seg, bus, path->dev, path->fn);
 			goto error;
+		}
 		path++;
 		count--;
 		bus = sec_bus;
@@ -187,14 +190,11 @@ error:
 }
 
 static int acpi_parse_dev_scope(fwts_framework *fw,
-	void *start, void *end, const uint16_t seg)
+	uint8_t *start, const uint8_t *end, const uint16_t seg)
 {
-	struct acpi_dev_scope *scope;
-	int ret;
-
 	while (start < end) {
-		scope = start;
-		ret = acpi_parse_one_dev_scope(fw, scope, seg);
+		struct acpi_dev_scope *scope = (struct acpi_dev_scope *)start;
+		int ret = acpi_parse_one_dev_scope(fw, scope, seg);
 		if (ret)
 			return ret;
 		start += scope->length;
@@ -205,7 +205,6 @@ static int acpi_parse_dev_scope(fwts_framework *fw,
 static int acpi_parse_one_drhd(fwts_framework *fw,
 	struct acpi_dmar_entry_header *header)
 {
-	static int include_all;
 	struct acpi_table_drhd *drhd = (struct acpi_table_drhd*)header;
 
 	if (drhd->address & MASK_4K) {
@@ -214,6 +213,8 @@ static int acpi_parse_one_drhd(fwts_framework *fw,
 		return FWTS_ERROR;
 	}
 	if (drhd->flags & 1) {
+		static int include_all;
+
 		if (include_all == 1) {
 			fwts_failed(fw, LOG_LEVEL_MEDIUM, "MultipleDRHDSFlag",
 				"Multiple drhds have include_all flag set.");
@@ -221,8 +222,8 @@ static int acpi_parse_one_drhd(fwts_framework *fw,
 		}
 		include_all = 1;
 	} else {
-		return acpi_parse_dev_scope(fw, (void *)(drhd + 1),
-			((void *)drhd) + header->length, drhd->segment);
+		return acpi_parse_dev_scope(fw, (uint8_t *)(drhd + 1),
+			((uint8_t *)drhd) + header->length, drhd->segment);
 	}
 	return FWTS_OK;
 }
@@ -239,8 +240,8 @@ static int acpi_parse_one_rmrr(fwts_framework *fw,
 			"Invalid rmrr range address.");
 		return FWTS_ERROR;
 	}
-	return acpi_parse_dev_scope(fw, (void *)(rmrr + 1),
-		((void*)rmrr) + header->length, rmrr->segment);
+	return acpi_parse_dev_scope(fw, (uint8_t *)(rmrr + 1),
+		((uint8_t *)rmrr) + header->length, rmrr->segment);
 }
 
 static int dmar_acpi_table_check(fwts_framework *fw)
@@ -264,7 +265,6 @@ static int dmar_acpi_table_check(fwts_framework *fw)
 	if (table->length <= DMAR_HEADER_SIZE) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "InvalidDMAR",
 			"Invalid DMAR ACPI table.");
-		fwts_tag_failed(fw, FWTS_TAG_ACPI_INVALID_TABLE);
 		return FWTS_ERROR;
 	}
 
@@ -281,7 +281,8 @@ static int dmar_acpi_table_check(fwts_framework *fw)
 			failed++;
 			break;
 		}
-		header = ((void *)header) + header->length;
+		header = (struct acpi_dmar_entry_header *)
+				(((char *)header) + header->length);
 	}
 
 	if (!failed)
@@ -293,6 +294,11 @@ static int dmar_acpi_table_check(fwts_framework *fw)
 static void acpiinfo_check(fwts_framework *fw,
 	char *line, int repeated, char *prevline, void *private, int *errors)
 {
+	FWTS_UNUSED(repeated);
+	FWTS_UNUSED(prevline);
+	FWTS_UNUSED(private);
+	FWTS_UNUSED(errors);
+
         if (strstr(line, "DMAR:[fault reason"))
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "DMARError",
 		"Found DMAR error: %s", line);
@@ -311,6 +317,7 @@ static int dmar_test1(fwts_framework *fw)
 		if (fwts_klog_scan(fw, klog, acpiinfo_check,
 			NULL, NULL, &errors)) {
                 	fwts_log_error(fw, "Failed to scan kernel log.");
+			fwts_klog_free(klog);
                 	return FWTS_ERROR;
 		}
 		if (errors == 0)
@@ -322,15 +329,15 @@ static int dmar_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test dmar_tests[] = {
-	{ dmar_test1, "Check DMA Remapping." },
+	{ dmar_test1, "DMA Remapping test." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops dmar_ops = {
-	.description = "Check sane DMA Remapping (VT-d).",
+	.description = "DMA Remapping (VT-d) test.",
 	.minor_tests = dmar_tests
 };
 
-FWTS_REGISTER(dmar, &dmar_ops, FWTS_TEST_ANYTIME, FWTS_BATCH_EXPERIMENTAL);
+FWTS_REGISTER("dmar", &dmar_ops, FWTS_TEST_ANYTIME, FWTS_FLAG_BATCH | FWTS_FLAG_ACPI | FWTS_FLAG_ROOT_PRIV)
 
 #endif
