@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2023 Canonical
+ * Copyright (C) 2015-2025 Canonical
  * Portions added (c) 2015, Al Stone <ahs3@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -122,7 +122,7 @@
 
 #define SUBTABLE_UNDEFINED	0x00
 #define SUBTABLE_VARIABLE	0xff
-#define NUM_SUBTABLE_TYPES	24
+#define NUM_SUBTABLE_TYPES	28
 #define MAX_IO_APIC_ID		256 /* IO APIC ID field is 1 byte */
 
 #define SBBR_ACPI_MAJOR_VERSION 6
@@ -247,6 +247,15 @@ static struct acpi_madt_subtable_lengths spec_info[] = {
 			     16, 16, 12, 82, 24, 24, 16, 20, 16,
 			     15, 23, 21, 13, 19, 17, 15 }
 	},
+	{ /* for ACPI 6.6 */
+		.major_version = 6,
+		.minor_version = 6,
+		.madt_version = 7,
+		.num_types = 28,
+		.lengths = { 8, 12, 10, 8, 6, 12, 16, SUBTABLE_VARIABLE,
+			     16, 16, 12, 82, 24, 24, 16, 20, 24,
+			     15, 23, 21, 13, 19, 17, 15, 36, 16, 36, 36 }
+	},
 	{ /* terminator */
 		.major_version = 0,
 		.minor_version = 0,
@@ -516,6 +525,9 @@ static int madt_arch_revision(fwts_framework *fw)
 	if (fw->target_arch == FWTS_ARCH_ARM64) {
 		minrev = 3;
 		arch = "aarch64";
+	} else if (fw->target_arch == FWTS_ARCH_RISCV64) {
+		minrev = 6;
+		arch = "riscv64";
 	} else {
 		minrev = 1;
 		arch = "intel";
@@ -575,10 +587,14 @@ static const char *madt_sub_names[] = {
 	/* 0x12 */ "LIO PIC",
 	/* 0x13 */ "HT PIC",
 	/* 0x14 */ "EIO PIC",
-	/* 0x16 */ "MSI PIC",
-	/* 0x17 */ "BIO PIC",
-	/* 0x18 */ "LPC PIC",
-	/* 0x11 - 0x7f */ "Reserved. OSPM skips structures of the reserved type.",
+	/* 0x15 */ "MSI PIC",
+	/* 0x16 */ "BIO PIC",
+	/* 0x17 */ "LPC PIC",
+	/* 0x18 */ "RINTC",
+	/* 0x19 */ "IMSIC",
+	/* 0x1a */ "APLIC",
+	/* 0x1b */ "PLIC",
+	/* 0x1c - 0x7f */ "Reserved. OSPM skips structures of the reserved type.",
 	/* 0x80 - 0xff */ "Reserved for OEM use",
 	NULL
 };
@@ -1069,6 +1085,9 @@ static int madt_gicc(fwts_framework *fw,
 {
 	/* specific checks for subtable type 0xb: GICC */
 	fwts_acpi_madt_gic *gic = (fwts_acpi_madt_gic *)data;
+	fwts_acpi_table_madt *madt = (fwts_acpi_table_madt *)mtable->data;
+	fwts_acpi_table_fadt *fadt = ftable ?
+		(fwts_acpi_table_fadt *)ftable->data : NULL;
 	uint32_t mask;
 	int start;
 
@@ -1083,7 +1102,8 @@ static int madt_gicc(fwts_framework *fw,
 			    "MADT %s reserved field properly set to zero.",
 			    madt_sub_names[hdr->type]);
 
-	madt_find_processor_uid(fw, gic->processor_uid, "GICC");
+	if((gic->flags & 1) != 0)
+		madt_find_processor_uid(fw, gic->processor_uid, "GICC");
 
 	mask = 0xfffffffc;
 	start = 2;
@@ -1091,6 +1111,10 @@ static int madt_gicc(fwts_framework *fw,
 		mask = 0xfffffff8;
 		start = 3;
 	}
+	if (hdr->length == 82) {	/* ACPI 6.5 */
+		mask = 0xfffffff0;
+		start = 4;
+	}
 	if (gic->flags & mask)
 		fwts_failed(fw, LOG_LEVEL_MEDIUM,
 			    "MADTGICFLags",
@@ -1103,19 +1127,65 @@ static int madt_gicc(fwts_framework *fw,
 			    "properly set to zero.",
 			    madt_sub_names[hdr->type], start);
 
-	if (gic->parking_protocol_version != 0 &&
-	    gic->parking_protocol_version != 1)
-		fwts_failed(fw, LOG_LEVEL_MEDIUM,
-			    "SPECMADTGICCParkingVersion",
-			    "MADT %s, protocol versions defined are 0..1 but "
-			    "%" PRIu32 " is being used.",
-			    madt_sub_names[hdr->type],
-			    gic->parking_protocol_version);
-	else
-		fwts_passed(fw,
-			    "MADT %s, is using a defined parking protocol "
-			    "version.",
-			    madt_sub_names[hdr->type]);
+	if (hdr->length == 82) {	/* ACPI 6.5 */
+		/* If the Enabled bit is set, Online Capable bit is reserved and must be zero. */
+		if ((gic->flags & 1) && (gic->flags & (1 << 3)))
+			fwts_failed(fw, LOG_LEVEL_MEDIUM,
+				    "MADTGICBadFLag",
+				    "MADT %s, flags, if Enable bit is set, the Online Capable bit "
+				    "is reserved and must be zero.",madt_sub_names[hdr->type]);
+	}
+
+	if (madt && madt->header.revision >= 7) {
+		if (gic->parking_protocol_version != 0)
+			fwts_failed(fw, LOG_LEVEL_MEDIUM,
+				    "SPECMADTGICCParkingVersion",
+				    "MADT revision %" PRIu8 " requires the Parking Protocol "
+				    "Version field to be zero, but MADT %s reports %" PRIu32 ".",
+				    madt->header.revision,
+				    madt_sub_names[hdr->type],
+				    gic->parking_protocol_version);
+		else
+			fwts_passed(fw,
+				    "MADT %s uses Parking Protocol Version 0 as "
+				    "required for MADT revision %" PRIu8 ".",
+				    madt_sub_names[hdr->type],
+				    madt->header.revision);
+
+		if (!fadt)
+			fwts_failed(fw, LOG_LEVEL_MEDIUM,
+				    "MADTArmPsciCompliance",
+				    "MADT revision %" PRIu8 " requires verifying PSCI compliance "
+				    "but the FADT table is not available.",
+				    madt->header.revision);
+		else if (!(fadt->arm_boot_flags & FWTS_FACP_ARM_BOOT_ARCH_PSCI_COMPLIANT))
+			fwts_failed(fw, LOG_LEVEL_MEDIUM,
+				    "MADTArmPsciCompliance",
+				    "MADT revision %" PRIu8 " requires PSCI to be implemented, "
+				    "but the FADT ARM Boot Architecture PSCI_COMPLIANT flag "
+				    "is not set (flags = 0x%4.4" PRIx16 ").",
+				    madt->header.revision,
+				    fadt->arm_boot_flags);
+		else
+			fwts_passed(fw,
+				    "FADT ARM Boot Architecture PSCI_COMPLIANT flag is set "
+				    "as required for MADT revision %" PRIu8 ".",
+				    madt->header.revision);
+	} else {
+		if (gic->parking_protocol_version != 0 &&
+		    gic->parking_protocol_version != 1)
+			fwts_failed(fw, LOG_LEVEL_MEDIUM,
+				    "SPECMADTGICCParkingVersion",
+				    "MADT %s, protocol versions defined are 0..1 but "
+				    "%" PRIu32 " is being used.",
+				    madt_sub_names[hdr->type],
+				    gic->parking_protocol_version);
+		else
+			fwts_passed(fw,
+				    "MADT %s is using a defined parking protocol "
+				    "version.",
+				    madt_sub_names[hdr->type]);
+	}
 
 	/*
 	 * TODO: is it even possible to verify the MPIDR is valid?  Or,
@@ -1298,23 +1368,50 @@ static int madt_gicr(fwts_framework *fw,
 {
 	/* specific checks for subtable type 0xe: GICR */
 	fwts_acpi_madt_gicr *gicr = (fwts_acpi_madt_gicr *)data;
+	fwts_acpi_table_madt *madt = (fwts_acpi_table_madt *)mtable->data;
 
 	/*
 	 * TODO: GICR structures should only be used when GICs implement
 	 * version 3 or higher.
 	 */
 
-	if (gicr->reserved)
-		fwts_failed(fw, LOG_LEVEL_LOW,
-			    "MADTGICRReservedNonZero",
-			    "MADT %s reserved field should be zero, "
-			    "instead got 0x%" PRIx32 ".",
-			    madt_sub_names[hdr->type],
-			    gicr->reserved);
-	else
-		fwts_passed(fw,
-			    "MADT %s reserved field properly set to zero.",
-			    madt_sub_names[hdr->type]);
+	if (madt && madt->header.revision >= 7) {
+		if (gicr->gicr.flags & ~0x1) {
+			fwts_failed(fw, LOG_LEVEL_LOW,
+				    "MADTGICRReservedFlags",
+				    "MADT %s flags bits 1..7 are reserved and should "
+				    "be zero, but 0x%" PRIx8 " was reported.",
+				    madt_sub_names[hdr->type], gicr->gicr.flags);
+		} else {
+			fwts_passed(fw,
+				    "MADT %s reserved flag bits are properly cleared.",
+				    madt_sub_names[hdr->type]);
+		}
+
+		if (gicr->gicr.reserved)
+			fwts_failed(fw, LOG_LEVEL_LOW,
+				    "MADTGICRReservedNonZero",
+				    "MADT %s reserved byte should be zero, "
+				    "instead got 0x%" PRIx8 ".",
+				    madt_sub_names[hdr->type],
+				    gicr->gicr.reserved);
+		else
+			fwts_passed(fw,
+				    "MADT %s reserved byte properly set to zero.",
+				    madt_sub_names[hdr->type]);
+	} else {
+		if (gicr->reserved)
+			fwts_failed(fw, LOG_LEVEL_LOW,
+				    "MADTGICRReservedNonZero",
+				    "MADT %s reserved field should be zero, "
+				    "instead got 0x%" PRIx16 ".",
+				    madt_sub_names[hdr->type],
+				    gicr->reserved);
+		else
+			fwts_passed(fw,
+				    "MADT %s reserved field properly set to zero.",
+				    madt_sub_names[hdr->type]);
+	}
 
 	/*
 	 * TODO: can Discovery Range Base Address ever be zero?
@@ -1341,20 +1438,46 @@ static int madt_gic_its(fwts_framework *fw,
 {
 	/* specific checks for subtable type 0xf: GIC ITS */
 	fwts_acpi_madt_gic_its *gic_its = (fwts_acpi_madt_gic_its *)data;
+	fwts_acpi_table_madt *madt = (fwts_acpi_table_madt *)mtable->data;
 	fwts_list_link *item;
 	bool found;
 
-	if (gic_its->reserved)
-		fwts_failed(fw, LOG_LEVEL_LOW,
-			    "SPECMADTGICITSReservedNonZero",
-			    "MADT %s first reserved field should be zero, "
-			    "instead got 0x%" PRIx32 ".",
-			    madt_sub_names[hdr->type], gic_its->reserved);
-	else
-		fwts_passed(fw,
-			    "MADT %s first reserved field is properly set "
-			    "to zero.",
-			    madt_sub_names[hdr->type]);
+	if (madt && madt->header.revision >= 7) {
+		if (gic_its->gic_its.flags & ~0x1)
+			fwts_failed(fw, LOG_LEVEL_LOW,
+				    "SPECMADTGICITSReservedFlags",
+				    "MADT %s flags bits 1..7 are reserved and should "
+				    "be zero, but 0x%" PRIx8 " was reported.",
+				    madt_sub_names[hdr->type], gic_its->gic_its.flags);
+		else
+			fwts_passed(fw,
+				    "MADT %s reserved flag bits are properly cleared.",
+				    madt_sub_names[hdr->type]);
+
+		if (gic_its->gic_its.reserved)
+			fwts_failed(fw, LOG_LEVEL_LOW,
+				    "SPECMADTGICITSReservedNonZero",
+				    "MADT %s reserved byte should be zero, "
+				    "instead got 0x%" PRIx8 ".",
+				    madt_sub_names[hdr->type], gic_its->gic_its.reserved);
+		else
+			fwts_passed(fw,
+				    "MADT %s reserved byte is properly set "
+				    "to zero.",
+				    madt_sub_names[hdr->type]);
+	} else {
+		if (gic_its->reserved)
+			fwts_failed(fw, LOG_LEVEL_LOW,
+				    "SPECMADTGICITSReservedNonZero",
+				    "MADT %s reserved field should be zero, "
+				    "instead got 0x%" PRIx16 ".",
+				    madt_sub_names[hdr->type], gic_its->reserved);
+		else
+			fwts_passed(fw,
+				    "MADT %s reserved field is properly set "
+				    "to zero.",
+				    madt_sub_names[hdr->type]);
+	}
 
 	/*
 	 * Check ITS ID against previously found IDs to see if it
@@ -1409,18 +1532,24 @@ static int madt_mp_wakup(fwts_framework *fw,
 			     uint8_t *data)
 {
 	/* specific checks for subtable type 0x10: Multiprocessor Wakeup */
-	fwts_acpi_madt_mp_wakeup *mp_wakeup = (fwts_acpi_madt_mp_wakeup *) data;
-
-	if (mp_wakeup->mail_box_version != 0)
+	fwts_acpi_madt_mp_wakeup *mp_wakeup = (fwts_acpi_madt_mp_wakeup *)data;
+	fwts_acpi_table_madt *madt = (fwts_acpi_table_madt *)mtable->data;
+	uint8_t madt_revision = madt ? madt->header.revision : 0;
+	uint32_t expected_version = (madt_revision >= 7) ? 1 : 0;
+	if (mp_wakeup->mail_box_version != expected_version) {
 		fwts_failed(fw, LOG_LEVEL_LOW,
 			    "SPECMADTMPWAKEUPVersion",
-			    "MADT %s mailbox version should be in 0, "
-			    "but instead have 0x%" PRIx32 ".",
-			    madt_sub_names[hdr->type], mp_wakeup->mail_box_version);
-	else
+			    "MADT %s mailbox version should be %" PRIu32
+			    " for MADT revision %" PRIu8 ", but instead has 0x%" PRIx32 ".",
+			    madt_sub_names[hdr->type],
+			    expected_version,
+			    madt_revision,
+			    mp_wakeup->mail_box_version);
+	} else {
 		fwts_passed(fw,
-			    "MADT %s mailbox version is in 0.",
-			    madt_sub_names[hdr->type]);
+			    "MADT %s mailbox version is set to %" PRIu32 ".",
+			    madt_sub_names[hdr->type], expected_version);
+	}
 
 	if (mp_wakeup->reserved)
 		fwts_failed(fw, LOG_LEVEL_LOW,
@@ -1442,11 +1571,9 @@ static int madt_mp_wakup(fwts_framework *fw,
 			    madt_sub_names[hdr->type], mp_wakeup->mail_box_address);
 	else
 		fwts_passed(fw,
-			    "MADT %s mailbox version is in 0.",
+			    "MADT %s mailbox address is 4K aligned.",
 			    madt_sub_names[hdr->type]);
 
-
-
 	return (hdr->length - sizeof(fwts_acpi_madt_sub_table_header));
 }
 
@@ -1547,6 +1674,280 @@ static void madt_ioapic_sapic_compare(fwts_framework *fw,
 			    "SAPIC entry.");
 }
 
+static int madt_rintc(fwts_framework *fw,
+		      fwts_acpi_madt_sub_table_header *hdr,
+		      uint8_t *data)
+{
+	/* specific checks for subtable type 0x18: RINTC */
+	fwts_acpi_madt_rintc *rintc = (fwts_acpi_madt_rintc *) data;
+
+	if (rintc->version != 1)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTRINTCVersion",
+			    "MADT %s version field should be 1, "
+			    "but instead got 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], rintc->version);
+	else
+		fwts_passed(fw,
+			    "MADT %s version field is properly set "
+			    "to 1.",
+			    madt_sub_names[hdr->type]);
+
+	if (rintc->reserved)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTRINTCReservedNonZero",
+			    "MADT %s reserved field should be 0, "
+			    "instead got 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], rintc->reserved);
+	else
+		fwts_passed(fw,
+			    "MADT %s reserved field is properly set "
+			    "to 0.",
+			    madt_sub_names[hdr->type]);
+
+	if (rintc->flags & 0xfffffffc)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTRINTCFlags",
+			    "MADT %s flags, bit 2..31 are reserved "
+			    "and should be 0, but are set as: 0x%" PRIx32 ".",
+			    madt_sub_names[hdr->type], rintc->flags);
+	else
+		fwts_passed(fw,
+			    "MADT %s flags, bit 2..31 are reserved "
+			    "and properly set to 0.",
+			    madt_sub_names[hdr->type]);
+
+	madt_find_processor_uid(fw, rintc->uid, "RINTC");
+
+	return (hdr->length - sizeof(fwts_acpi_madt_sub_table_header));
+}
+
+static int madt_imsic(fwts_framework *fw,
+		      fwts_acpi_madt_sub_table_header *hdr,
+		      uint8_t *data)
+{
+	/* specific checks for subtable type 0x19: IMSIC */
+	fwts_acpi_madt_imsic *imsic = (fwts_acpi_madt_imsic *) data;
+
+	if (imsic->version != 1)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICVersion",
+			    "MADT %s version field should be 1, "
+			    "but instead got 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], imsic->version);
+	else
+		fwts_passed(fw,
+			    "MADT %s version field is properly set "
+			    "to 1.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->reserved)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "SPECMADTMIMSICReservedNonZero",
+			    "MADT %s reserved field should be 0, "
+			    "instead got 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], imsic->reserved);
+	else
+		fwts_passed(fw,
+			    "MADT %s reserved field is properly set "
+			    "to 0.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->flags)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICFlags",
+			    "MADT %s flags field should be 0, "
+			    "but are set as: 0x%" PRIx32 ".",
+			    madt_sub_names[hdr->type], imsic->flags);
+	else
+		fwts_passed(fw,
+			    "MADT %s flags field should be 0 "
+			    "and properly set to 0.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->num_ids < 63 || imsic->num_ids > 2047)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICNumIDs",
+			    "MADT %s num_ids field should be 63..2047, "
+			    "but are set as: 0x%" PRIx16 ".",
+			    madt_sub_names[hdr->type], imsic->num_ids);
+	else
+		fwts_passed(fw,
+			    "MADT %s num_ids field should be 63..2047 "
+			    "and properly set.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->num_guest_ids < 63 || imsic->num_guest_ids > 2047)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICNumGuestIDs",
+			    "MADT %s num_guest_ids field should be 63..2047, "
+			    "but are set as: 0x%" PRIx16 ".",
+			    madt_sub_names[hdr->type], imsic->num_guest_ids);
+	else
+		fwts_passed(fw,
+			    "MADT %s num_guest_ids field should be 63..2047 "
+			    "and properly set.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->guest_index_bits > 7)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICGuestIndexBits",
+			    "MADT %s guest_index_bits field should be 0..7, "
+			    "but are set as: 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], imsic->guest_index_bits);
+	else
+		fwts_passed(fw,
+			    "MADT %s guest_index_bits field should be 0..7 "
+			    "and properly set.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->hart_index_bits > 15)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICHartIndexBits",
+			    "MADT %s hart_index_bits field should be 0..15, "
+			    "but are set as: 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], imsic->hart_index_bits);
+	else
+		fwts_passed(fw,
+			    "MADT %s hart_index_bits field should be 0..15 "
+			    "and properly set.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->group_index_bits > 7)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICGroupIndexBits",
+			    "MADT %s group_index_bits field should be 0..7, "
+			    "but are set as: 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], imsic->group_index_bits);
+	else
+		fwts_passed(fw,
+			    "MADT %s group_index_bits field should be 0..7 "
+			    "and properly set.",
+			    madt_sub_names[hdr->type]);
+
+	if (imsic->group_index_shift > 55)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTIMSICGroupIndexShift",
+			    "MADT %s group_index_shift field should be 0..55, "
+			    "but are set as: 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], imsic->group_index_shift);
+	else
+		fwts_passed(fw,
+			    "MADT %s group_index_shift field should be 0..55 "
+			    "and properly set.",
+			    madt_sub_names[hdr->type]);
+
+	return (hdr->length - sizeof(fwts_acpi_madt_sub_table_header));
+}
+
+static int madt_aplic(fwts_framework *fw,
+		      fwts_acpi_madt_sub_table_header *hdr,
+		      uint8_t *data)
+{
+	/* specific checks for subtable type 0x1a: APLIC */
+	fwts_acpi_madt_aplic *aplic = (fwts_acpi_madt_aplic *) data;
+
+	if (aplic->version != 1)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTAPLICVersion",
+			    "MADT %s version field should be 1, "
+			    "but instead got 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], aplic->version);
+	else
+		fwts_passed(fw,
+			    "MADT %s version field is properly set "
+			    "to 1.",
+			    madt_sub_names[hdr->type]);
+
+	if (aplic->flags)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTAPLICFlags",
+			    "MADT %s flags field should be 0, "
+			    "but are set as: 0x%" PRIx32 ".",
+			    madt_sub_names[hdr->type], aplic->flags);
+	else
+		fwts_passed(fw,
+			    "MADT %s flags field should be 0 "
+			    "and properly set to 0.",
+			    madt_sub_names[hdr->type]);
+
+	if (aplic->num_sources < 1 || aplic->num_sources > 1023)
+		fwts_failed(fw, LOG_LEVEL_MEDIUM,
+			    "MADTAPLICNumSources",
+			    "MADT %s num_sources field is %" PRIu16 ", must be 1..1023.",
+			    madt_sub_names[hdr->type], aplic->num_sources);
+	else
+		fwts_passed(fw,
+			    "MADT %s num_sources field is %" PRIu16 " and in 1..1023.",
+			    madt_sub_names[hdr->type], aplic->num_sources);
+
+	if (aplic->addr == 0)
+		fwts_failed(fw, LOG_LEVEL_MEDIUM,
+			    "MADTAPLICADDR",
+			    "MADT %s addr field is 0, appears not defined.",
+			    madt_sub_names[hdr->type]);
+	else
+		fwts_passed(fw,
+			    "MADT %s addr field is properly defined.",
+			    madt_sub_names[hdr->type]);
+
+	return (hdr->length - sizeof(fwts_acpi_madt_sub_table_header));
+}
+
+static int madt_plic(fwts_framework *fw,
+		      fwts_acpi_madt_sub_table_header *hdr,
+		      uint8_t *data)
+{
+	/* specific checks for subtable type 0x1b: PLIC */
+	fwts_acpi_madt_plic *plic = (fwts_acpi_madt_plic *) data;
+
+	if (plic->version != 1)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTPLICVersion",
+			    "MADT %s version field should be 1, "
+			    "but instead got 0x%" PRIx8 ".",
+			    madt_sub_names[hdr->type], plic->version);
+	else
+		fwts_passed(fw,
+			    "MADT %s version field is properly set "
+			    "to 1.",
+			    madt_sub_names[hdr->type]);
+
+	if (plic->num_irqs < 1 || plic->num_irqs > 1023)
+		fwts_failed(fw, LOG_LEVEL_MEDIUM,
+			    "MADTPLICNumIrqs",
+			    "MADT %s num_irqs field is %" PRIu16 ", must be 1..1023.",
+			    madt_sub_names[hdr->type], plic->num_irqs);
+	else
+		fwts_passed(fw,
+			    "MADT %s num_irqs field is %" PRIu16 " and in 1..1023.",
+			    madt_sub_names[hdr->type], plic->num_irqs);
+
+	if (plic->flags)
+		fwts_failed(fw, LOG_LEVEL_LOW,
+			    "MADTPLICFlags",
+			    "MADT %s flags field should be 0, "
+			    "but are set as: 0x%" PRIx32 ".",
+			    madt_sub_names[hdr->type], plic->flags);
+	else
+		fwts_passed(fw,
+			    "MADT %s flags field should be 0 "
+			    "and properly set to 0.",
+			    madt_sub_names[hdr->type]);
+
+	if (plic->base_addr == 0)
+		fwts_failed(fw, LOG_LEVEL_MEDIUM,
+			    "MADTPLICADDR",
+			    "MADT %s base_addr field is 0, appears not defined.",
+			    madt_sub_names[hdr->type]);
+	else
+		fwts_passed(fw,
+			    "MADT %s base_addr field is properly defined.",
+			    madt_sub_names[hdr->type]);
+
+	return (hdr->length - sizeof(fwts_acpi_madt_sub_table_header));
+}
+
 static int madt_subtables(fwts_framework *fw)
 {
 	fwts_acpi_table_madt *madt = (fwts_acpi_table_madt *)mtable->data;
@@ -1743,6 +2144,22 @@ static int madt_subtables(fwts_framework *fw)
 			skip = madt_general_pic(fw, hdr, data);
 			break;
 
+		case FWTS_MADT_RINTC:
+			skip = madt_rintc(fw, hdr, data);
+			break;
+
+		case FWTS_MADT_IMSIC:
+			skip = madt_imsic(fw, hdr, data);
+			break;
+
+		case FWTS_MADT_APLIC:
+			skip = madt_aplic(fw, hdr, data);
+			break;
+
+		case FWTS_MADT_PLIC:
+			skip = madt_plic(fw, hdr, data);
+			break;
+
 		case FWTS_MADT_RESERVED:
 			fwts_failed(fw, LOG_LEVEL_MEDIUM,
 				    "SPECMADTSubReservedID",
