@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2023 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +31,7 @@
 #include <linux/rtc.h>
 
 static struct rtc_time rtc_tm;
+static bool skip_suite = false;
 
 static int wakealarm_test1(fwts_framework *fw)
 {
@@ -47,6 +48,8 @@ static int wakealarm_test1(fwts_framework *fw)
 			"does not exist, so the wake alarm tests will be aborted.");
 		return FWTS_ABORTED;
 #else
+		skip_suite = true;
+
 		fwts_log_info(fw,
 			"non-x86 devices sometimes do not have an RTC wake alarm that "
 			"is normally controlled by the RTC alarm ioctl() interface. This "
@@ -59,6 +62,10 @@ static int wakealarm_test1(fwts_framework *fw)
 
 static int wakealarm_test2(fwts_framework *fw)
 {
+	if (skip_suite) {
+		return FWTS_SKIP;
+	}
+
 	fwts_log_info(fw, "Trigger wakealarm for 1 seconds in the future.");
 	if (fwts_wakealarm_trigger(fw, 1)) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "WakeAlarmNotTriggeredTest2",
@@ -77,6 +84,10 @@ static int wakealarm_test3(fwts_framework *fw)
 {
 	int ret;
 
+	if (skip_suite) {
+		return FWTS_SKIP;
+	}
+
 	ret = fwts_wakealarm_test_firing(fw, 2);
 	if (ret < 0) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "WakeAlarmNotTriggeredTest3",
@@ -98,6 +109,10 @@ static int wakealarm_test4(fwts_framework *fw)
 	uint32_t i;
 	int failed = 0;
 
+	if (skip_suite) {
+		return FWTS_SKIP;
+	}
+
 	for (i = 1; i < 5; i++) {
 		fwts_log_info(fw, "Trigger wakealarm for %" PRIu32 " seconds in the future.", i);
 		int ret = fwts_wakealarm_test_firing(fw, i);
@@ -125,6 +140,10 @@ static int wakealarm_test5(fwts_framework *fw)
 {
 	struct rtc_time rtc_now;
 
+	if (skip_suite) {
+		return FWTS_SKIP;
+	}
+
 	if (fwts_wakealarm_set(fw, &rtc_tm) == FWTS_OK) {
 		fwts_passed(fw, "RTC wakealarm set.");
 	} else {
@@ -146,6 +165,14 @@ static int wakealarm_test5(fwts_framework *fw)
 					rtc_tm.tm_mday, rtc_tm.tm_mon + 1,
 					rtc_tm.tm_year + 1900, rtc_tm.tm_hour,
 					rtc_tm.tm_min, rtc_tm.tm_sec);
+		} else if (rtc_now.tm_hour == rtc_tm.tm_hour &&
+		    rtc_now.tm_min == rtc_tm.tm_min &&
+		    rtc_now.tm_sec == rtc_tm.tm_sec) {
+			fwts_skipped(fw,
+				"The kernel adjusted the RTC_ALM_SET alarm by up to 24 hours "
+				"because the requested time was earlier than the current time. "
+				"This behavior ensures the alarm always points to a future time.");
+			return FWTS_SKIP;
 		} else {
 			fwts_failed(fw, LOG_LEVEL_MEDIUM,
 				"WakeAlarmNotResetTest5",
