@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2023 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -93,6 +93,7 @@ static fwts_wmi_known_guid fwts_wmi_known_guids[] = {
 	{ "F1DDEE52-063C-4784-A11E-8A06684B9BF4", "dell-wmi-sysman", "Dell" },
 	{ "70FE8229-D03B-4214-A1C6-1F884B1A892A", "dell-wmi-sysman", "Dell" },
 	{ "8A42EA14-4F2A-FD45-6422-0087F7A7E608", "dell-wmi-ddv", "Dell" },
+	{ "F1DDEE52-063C-4784-A11E-8A06684B9B01", "dell-smm-hwmon", "Dell" },
 	{ "ABBC0F72-8EA1-11D1-00A0-C90629100000", "eeepc-wmi",	"Asus" },
 	{ "97845ED0-4E6D-11DE-8A39-0800200C9A66", "eeepc-wmi",	"Asus" },
 	{ "DEADBEEF-2001-0000-00A0-C90629100000", "gigabyte-wmi",	"Gigabyte" },
@@ -128,6 +129,25 @@ static fwts_wmi_known_guid fwts_wmi_known_guids[] = {
 	{ NULL, NULL, NULL }
 };
 
+/*
+ *  List of WMI GUIDs used inside the Windows driver samples.
+ *
+ *  Some manufacturers just blindly copy those, which prevents their WMI drivers
+ *  from being loaded automatically since those GUIDs are not unique and cannot
+ *  be used for reliable WMI device identification.
+ */
+static const char * const windows_example_wmi_guids[] = {
+	"ABBC0F6A-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F6B-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F6C-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F6D-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F6E-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F6F-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F70-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F71-8EA1-11D1-00A0-C90629100000",
+	"ABBC0F72-8EA1-11D1-00A0-C90629100000"
+};
+
 /*
  *  WMI flag to text mappings
  */
@@ -239,7 +259,7 @@ static char *wmi_acpi_get_parent_name(const char *object_name)
 
 /*
  * wmi_acpi_method_count_on_object()
- *	count occurance of methods on acpi object
+ *	count occurrence of methods on acpi object
  */
 static int wmi_acpi_method_count_on_object(const char *object_name, const char *method_name)
 {
@@ -340,6 +360,31 @@ static void wmi_method_exist_count(
 			guid_str, object_name, wm_name);
 }
 
+/*
+ *  wmi_guid_copycat()
+ *	Warn if a WMI from the Windows driver samples was found. This usually means that
+ *	the manufacturer just blindly copied the example code without generating new
+ *	unique WMI GUIDs.
+ */
+static void wmi_guid_copycat(
+	fwts_framework *fw,
+	const char *guid_str)
+{
+	int i;
+
+	for (i = 0; i < FWTS_ARRAY_SIZE(windows_example_wmi_guids); i++) {
+		if (strcmp(guid_str, windows_example_wmi_guids[i]) == 0) {
+			fwts_failed(fw, LOG_LEVEL_MEDIUM,
+				"WMIExampleGUIDCopycat",
+				"GUID %s has likely been copied from the Windows driver samples, "
+				"this is a firmware bug that prevents WMI drivers from reliably "
+				"detecting such WMI devices since their GUID is not unique.",
+				guid_str);
+			return;
+		}
+	}
+}
+
 /*
  *  wmi_no_known_driver()
  *	grumble that the kernel does not have a known handler for this GUID
@@ -357,13 +402,9 @@ static void wmi_no_known_driver(
 	if (!wmi_advice_given) {
 		wmi_advice_given = true;
 		fwts_log_advice(fw,
-			"A WMI driver probably needs to be written for this "
-			"WMI event. It can checked for using: wmi_has_guid(\"%s\"). "
-			"One can install a notify handler using "
-			"wmi_install_notify_handler(\"%s\", handler, NULL).  "
-			"http://lwn.net/Articles/391230 describes how to write an "
-			"appropriate driver.",
-			guid_str, guid_str);
+			"A WMI driver probably needs to be written for this WMI event. "
+			"https://docs.kernel.org/wmi/driver-development-guide.html describes how "
+			"to write an appropriate WMI driver.");
 	}
 }
 
@@ -460,6 +501,14 @@ static void wmi_parse_wdg_data(
 			wmi_known_driver(fw, known);
 			wmi_block_query_exist_count(fw, info, acpi_object_name, guid_str);
 		}
+
+		wmi_guid_copycat(fw, guid_str);
+
+		if (info->instance == 0)
+			fwts_failed(fw, LOG_LEVEL_LOW, "WMIZeroInstance",
+				    "GUID %s has zero instances", guid_str);
+		else
+			fwts_passed(fw, "%s has more than zero instances", guid_str);
 	}
 
 	if (events && all_events_known)
