@@ -8,13 +8,13 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2010, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2025, Intel Corp.
  * All rights reserved.
  *
  * 2. License
  *
  * 2.1. This is your license from Intel Corp. under its intellectual property
- * rights.  You may have additional license terms from the party that provided
+ * rights. You may have additional license terms from the party that provided
  * you this software, covering your right to use that party's intellectual
  * property rights.
  *
@@ -31,7 +31,7 @@
  * offer to sell, and import the Covered Code and derivative works thereof
  * solely to the minimum extent necessary to exercise the above copyright
  * license, and in no event shall the patent license extend to any additions
- * to or modifications of the Original Intel Code.  No other license or right
+ * to or modifications of the Original Intel Code. No other license or right
  * is granted directly or by implication, estoppel or otherwise;
  *
  * The above copyright and patent license is granted only if the following
@@ -43,11 +43,11 @@
  * Redistribution of source code of any substantial portion of the Covered
  * Code or modification with rights to further distribute source must include
  * the above Copyright Notice, the above License, this list of Conditions,
- * and the following Disclaimer and Export Compliance provision.  In addition,
+ * and the following Disclaimer and Export Compliance provision. In addition,
  * Licensee must cause all Covered Code to which Licensee contributes to
  * contain a file documenting the changes Licensee made to create that Covered
- * Code and the date of any change.  Licensee must include in that file the
- * documentation of any changes made by any predecessor Licensee.  Licensee
+ * Code and the date of any change. Licensee must include in that file the
+ * documentation of any changes made by any predecessor Licensee. Licensee
  * must include a prominent statement that the modification is derived,
  * directly or indirectly, from Original Intel Code.
  *
@@ -55,7 +55,7 @@
  * Redistribution of source code of any substantial portion of the Covered
  * Code or modification without rights to further distribute source must
  * include the following Disclaimer and Export Compliance provision in the
- * documentation and/or other materials provided with distribution.  In
+ * documentation and/or other materials provided with distribution. In
  * addition, Licensee may not authorize further sublicense of source of any
  * portion of the Covered Code, and must include terms to the effect that the
  * license from Licensee to its licensee is limited to the intellectual
@@ -80,10 +80,10 @@
  * 4. Disclaimer and Export Compliance
  *
  * 4.1. INTEL MAKES NO WARRANTY OF ANY KIND REGARDING ANY SOFTWARE PROVIDED
- * HERE.  ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
- * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT,  ASSISTANCE,
- * INSTALLATION, TRAINING OR OTHER SERVICES.  INTEL WILL NOT PROVIDE ANY
- * UPDATES, ENHANCEMENTS OR EXTENSIONS.  INTEL SPECIFICALLY DISCLAIMS ANY
+ * HERE. ANY SOFTWARE ORIGINATING FROM INTEL OR DERIVED FROM INTEL SOFTWARE
+ * IS PROVIDED "AS IS," AND INTEL WILL NOT PROVIDE ANY SUPPORT, ASSISTANCE,
+ * INSTALLATION, TRAINING OR OTHER SERVICES. INTEL WILL NOT PROVIDE ANY
+ * UPDATES, ENHANCEMENTS OR EXTENSIONS. INTEL SPECIFICALLY DISCLAIMS ANY
  * IMPLIED WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT AND FITNESS FOR A
  * PARTICULAR PURPOSE.
  *
@@ -92,14 +92,14 @@
  * COSTS OF PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, OR FOR ANY INDIRECT,
  * SPECIAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THIS AGREEMENT, UNDER ANY
  * CAUSE OF ACTION OR THEORY OF LIABILITY, AND IRRESPECTIVE OF WHETHER INTEL
- * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES.  THESE LIMITATIONS
+ * HAS ADVANCE NOTICE OF THE POSSIBILITY OF SUCH DAMAGES. THESE LIMITATIONS
  * SHALL APPLY NOTWITHSTANDING THE FAILURE OF THE ESSENTIAL PURPOSE OF ANY
  * LIMITED REMEDY.
  *
  * 4.3. Licensee shall not export, either directly or indirectly, any of this
  * software or system incorporating such software without first obtaining any
  * required license or other approval from the U. S. Department of Commerce or
- * any other agency or department of the United States Government.  In the
+ * any other agency or department of the United States Government. In the
  * event Licensee exports any such software from the United States or
  * re-exports any such software from a foreign destination, Licensee shall
  * ensure that the distribution and export/re-export of the software is in
@@ -111,12 +111,45 @@
  * other governmental approval, or letter of assurance, without first obtaining
  * such license, approval or letter.
  *
+ *****************************************************************************
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * following license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions, and the following disclaimer,
+ *    without modification.
+ * 2. Redistributions in binary form must reproduce at minimum a disclaimer
+ *    substantially similar to the "NO WARRANTY" disclaimer below
+ *    ("Disclaimer") and any redistribution must be conditioned upon
+ *    including a substantially similar Disclaimer requirement for further
+ *    binary redistribution.
+ * 3. Neither the names of the above-listed copyright holders nor the names
+ *    of any contributors may be used to endorse or promote products derived
+ *    from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Alternatively, you may choose to be licensed under the terms of the
+ * GNU General Public License ("GPL") version 2 as published by the Free
+ * Software Foundation.
+ *
  *****************************************************************************/
 
-#define __DTUTILS_C__
-
 #include "aslcompiler.h"
-#include "dtcompiler.h"
 #include "actables.h"
 
 #define _COMPONENT          DT_COMPILER
@@ -149,23 +182,26 @@ DtSum (
 void
 DtError (
     UINT8                   Level,
-    UINT8                   MessageId,
+    UINT16                  MessageId,
     DT_FIELD                *FieldObject,
     char                    *ExtraMessage)
 {
+    UINT32                  Line = 0;
 
-    switch (Level)
+
+    /* Field object could be NULL */
+
+    if (FieldObject)
     {
-    case ASL_WARNING2:
-    case ASL_WARNING3:
-        if (Gbl_WarningLevel < Level)
-        {
-            return;
-        }
-        break;
+        Line = FieldObject->Line;
+    }
 
-    default:
-        break;
+    /* Check if user wants to ignore this exception */
+
+    if (AslIsExceptionIgnored (AslGbl_Files[ASL_FILE_INPUT].Filename,
+        Line, Level, MessageId))
+    {
+        return;
     }
 
     if (FieldObject)
@@ -175,7 +211,7 @@ DtError (
             FieldObject->Line,
             FieldObject->ByteOffset,
             FieldObject->Column,
-            Gbl_Files[ASL_FILE_INPUT].Filename, ExtraMessage);
+            AslGbl_Files[ASL_FILE_INPUT].Filename, ExtraMessage);
     }
     else
     {
@@ -203,7 +239,7 @@ DtError (
 void
 DtNameError (
     UINT8                   Level,
-    UINT8                   MessageId,
+    UINT16                  MessageId,
     DT_FIELD                *FieldObject,
     char                    *ExtraMessage)
 {
@@ -212,13 +248,15 @@ DtNameError (
     {
     case ASL_WARNING2:
     case ASL_WARNING3:
-        if (Gbl_WarningLevel < Level)
+
+        if (AslGbl_WarningLevel < Level)
         {
             return;
         }
         break;
 
     default:
+
         break;
     }
 
@@ -229,7 +267,7 @@ DtNameError (
             FieldObject->Line,
             FieldObject->ByteOffset,
             FieldObject->NameColumn,
-            Gbl_Files[ASL_FILE_INPUT].Filename, ExtraMessage);
+            AslGbl_Files[ASL_FILE_INPUT].Filename, ExtraMessage);
     }
     else
     {
@@ -254,141 +292,63 @@ DtNameError (
 
 void
 DtFatal (
-    UINT8                   MessageId,
+    UINT16                  MessageId,
     DT_FIELD                *FieldObject,
     char                    *ExtraMessage)
 {
 
     DtError (ASL_ERROR, MessageId, FieldObject, ExtraMessage);
 
+/*
+ * TBD: remove this entire function, DtFatal
+ *
+ * We cannot abort the compiler on error, because we may be compiling a
+ * list of files. We must move on to the next file.
+ */
+#ifdef __OBSOLETE
     CmCleanupAndExit ();
     exit (1);
+#endif
 }
 
 
-/******************************************************************************
+/*******************************************************************************
  *
- * FUNCTION:    DtStrtoul64
+ * FUNCTION:    DtDoConstant
  *
- * PARAMETERS:  String              - Null terminated string
- *              ReturnInteger       - Where the converted integer is returned
+ * PARAMETERS:  String              - Only hex constants are supported,
+ *                                    regardless of whether the 0x prefix
+ *                                    is used
  *
- * RETURN:      Status
+ * RETURN:      Converted Integer
  *
- * DESCRIPTION: Simple conversion of a string hex integer constant to unsigned
- *              value. Assumes no leading "0x" for the constant.
+ * DESCRIPTION: Convert a string to an integer, with overflow/error checking.
  *
- * Portability note: The reason this function exists is because a 64-bit
- * sscanf is not available in all environments.
- *
- *****************************************************************************/
+ ******************************************************************************/
 
-ACPI_STATUS
-DtStrtoul64 (
-    char                    *String,
-    UINT64                  *ReturnInteger)
+UINT64
+DtDoConstant (
+    char                    *String)
 {
-    char                    *ThisChar = String;
-    UINT32                  ThisDigit;
-    UINT64                  ReturnValue = 0;
-    int                     DigitCount = 0;
+    UINT64                  ConvertedInteger;
 
 
-    /* Skip over any white space in the buffer */
-
-    while ((*ThisChar == ' ') || (*ThisChar == '\t'))
-    {
-        ThisChar++;
-    }
-
-    /* Skip leading zeros */
-
-    while ((*ThisChar) == '0')
-    {
-        ThisChar++;
-    }
-
-    /* Convert character-by-character */
-
-    while (*ThisChar)
-    {
-        if (ACPI_IS_DIGIT (*ThisChar))
-        {
-            /* Convert ASCII 0-9 to Decimal value */
-
-            ThisDigit = ((UINT8) *ThisChar) - '0';
-        }
-        else /* Letter */
-        {
-            ThisDigit = (UINT32) ACPI_TOUPPER (*ThisChar);
-            if (!ACPI_IS_XDIGIT ((char) ThisDigit))
-            {
-                /* Not A-F */
-
-                return (AE_BAD_CHARACTER);
-            }
-
-            /* Convert ASCII Hex char (A-F) to value */
-
-            ThisDigit = (ThisDigit - 'A') + 10;
-        }
-
-        /* Insert the 4-bit hex digit */
-
-        ReturnValue <<= 4;
-        ReturnValue += ThisDigit;
-
-        ThisChar++;
-        DigitCount++;
-        if (DigitCount > 16)
-        {
-            /* Value is too large (> 64 bits/8 bytes/16 hex digits) */
-
-            return (AE_LIMIT);
-        }
-    }
-
-    *ReturnInteger = ReturnValue;
-    return (AE_OK);
+    /*
+     * TBD: The ImplicitStrtoul64 function does not report overflow
+     * conditions. The input string is simply truncated. If it is
+     * desired to report overflow to the table compiler, this should
+     * somehow be added here. Note: integers that are prefixed with 0x
+     * or not are both hex integers.
+     */
+    ConvertedInteger = AcpiUtImplicitStrtoul64 (String);
+    return (ConvertedInteger);
 }
 
-
-/******************************************************************************
- *
- * FUNCTION:    DtGetFileSize
- *
- * PARAMETERS:  Handle              - Open file handler
- *
- * RETURN:      Current file size
- *
- * DESCRIPTION: Get the current size of a file. Seek to the EOF and get the
- *              offset. Seek back to the original location.
- *
- *****************************************************************************/
-
-UINT32
-DtGetFileSize (
-    FILE                    *Handle)
-{
-    int                     CurrentOffset;
-    int                     LastOffset;
-
-
-    CurrentOffset = ftell (Handle);
-    fseek (Handle, 0, SEEK_END);
-    LastOffset = ftell (Handle);
-    fseek (Handle, CurrentOffset, SEEK_SET);
-
-    return ((UINT32) LastOffset);
-}
-
-
 /******************************************************************************
  *
  * FUNCTION:    DtGetFieldValue
  *
  * PARAMETERS:  Field               - Current field list pointer
- *              Name                - Field name
  *
  * RETURN:      Field value
  *
@@ -398,23 +358,14 @@ DtGetFileSize (
 
 char *
 DtGetFieldValue (
-    DT_FIELD                *Field,
-    char                    *Name)
+    DT_FIELD                *Field)
 {
-
-    /* Search the field list for the name */
-
-    while (Field)
+    if (!Field)
     {
-        if (!ACPI_STRCMP (Name, Field->Name))
-        {
-            return (Field->Value);
-        }
-
-        Field = Field->Next;
+        return (NULL);
     }
 
-    return (NULL);
+    return (Field->Value);
 }
 
 
@@ -458,7 +409,16 @@ DtGetFieldType (
     case ACPI_DMT_FLAG6:
     case ACPI_DMT_FLAG7:
     case ACPI_DMT_FLAGS0:
+    case ACPI_DMT_FLAGS1:
     case ACPI_DMT_FLAGS2:
+    case ACPI_DMT_FLAGS8_2:
+    case ACPI_DMT_FLAGS4:
+    case ACPI_DMT_FLAGS4_0:
+    case ACPI_DMT_FLAGS4_4:
+    case ACPI_DMT_FLAGS4_8:
+    case ACPI_DMT_FLAGS4_12:
+    case ACPI_DMT_FLAGS16_16:
+
         Type = DT_FIELD_TYPE_FLAG;
         break;
 
@@ -467,21 +427,60 @@ DtGetFieldType (
     case ACPI_DMT_NAME6:
     case ACPI_DMT_NAME8:
     case ACPI_DMT_STRING:
+    case ACPI_DMT_IVRS_UNTERMINATED_STRING:
+
         Type = DT_FIELD_TYPE_STRING;
         break;
 
     case ACPI_DMT_BUFFER:
+    case ACPI_DMT_RAW_BUFFER:
+    case ACPI_DMT_BUF7:
+    case ACPI_DMT_BUF10:
+    case ACPI_DMT_BUF11:
+    case ACPI_DMT_BUF12:
     case ACPI_DMT_BUF16:
+    case ACPI_DMT_BUF18:
+    case ACPI_DMT_BUF24:
+    case ACPI_DMT_BUF26:
+    case ACPI_DMT_BUF32:
+    case ACPI_DMT_BUF112:
+    case ACPI_DMT_BUF128:
     case ACPI_DMT_PCI_PATH:
+    case ACPI_DMT_PMTT_VENDOR:
+
         Type = DT_FIELD_TYPE_BUFFER;
         break;
 
     case ACPI_DMT_GAS:
     case ACPI_DMT_HESTNTFY:
+    case ACPI_DMT_IORTMEM:
+
         Type = DT_FIELD_TYPE_INLINE_SUBTABLE;
         break;
 
+    case ACPI_DMT_UNICODE:
+    case ACPI_DMT_WPBT_UNICODE:
+
+        Type = DT_FIELD_TYPE_UNICODE;
+        break;
+
+    case ACPI_DMT_UUID:
+
+        Type = DT_FIELD_TYPE_UUID;
+        break;
+
+    case ACPI_DMT_DEVICE_PATH:
+
+        Type = DT_FIELD_TYPE_DEVICE_PATH;
+        break;
+
+    case ACPI_DMT_LABEL:
+
+        Type = DT_FIELD_TYPE_LABEL;
+        break;
+
     default:
+
         Type = DT_FIELD_TYPE_INTEGER;
         break;
     }
@@ -533,7 +532,7 @@ DtGetBufferLength (
  *
  * FUNCTION:    DtGetFieldLength
  *
- * PARAMETERS:  Field               - Current field list pointer
+ * PARAMETERS:  Field               - Current field
  *              Info                - Data table info
  *
  * RETURN:      Field length
@@ -566,7 +565,18 @@ DtGetFieldLength (
     case ACPI_DMT_FLAG6:
     case ACPI_DMT_FLAG7:
     case ACPI_DMT_FLAGS0:
+    case ACPI_DMT_FLAGS1:
     case ACPI_DMT_FLAGS2:
+    case ACPI_DMT_FLAGS8_2:
+    case ACPI_DMT_FLAGS4:
+    case ACPI_DMT_FLAGS4_0:
+    case ACPI_DMT_FLAGS4_4:
+    case ACPI_DMT_FLAGS4_8:
+    case ACPI_DMT_FLAGS4_12:
+    case ACPI_DMT_FLAGS16_16:
+    case ACPI_DMT_LABEL:
+    case ACPI_DMT_EXTRA_TEXT:
+
         ByteLength = 0;
         break;
 
@@ -574,67 +584,141 @@ DtGetFieldLength (
     case ACPI_DMT_CHKSUM:
     case ACPI_DMT_SPACEID:
     case ACPI_DMT_ACCWIDTH:
+    case ACPI_DMT_CEDT:
     case ACPI_DMT_IVRS:
+    case ACPI_DMT_IVRS_DE:
+    case ACPI_DMT_GTDT:
     case ACPI_DMT_MADT:
+    case ACPI_DMT_MPAM_LOCATOR:
+    case ACPI_DMT_PCCT:
+    case ACPI_DMT_PMTT:
+    case ACPI_DMT_PPTT:
+    case ACPI_DMT_RGRT:
+    case ACPI_DMT_SDEV:
     case ACPI_DMT_SRAT:
+    case ACPI_DMT_AEST:
+    case ACPI_DMT_AEST_RES:
+    case ACPI_DMT_AEST_XFACE:
+    case ACPI_DMT_AEST_XRUPT:
     case ACPI_DMT_ASF:
+    case ACPI_DMT_CDAT:
     case ACPI_DMT_HESTNTYP:
     case ACPI_DMT_FADTPM:
     case ACPI_DMT_EINJACT:
     case ACPI_DMT_EINJINST:
     case ACPI_DMT_ERSTACT:
     case ACPI_DMT_ERSTINST:
+    case ACPI_DMT_DMAR_SCOPE:
+    case ACPI_DMT_VIOT:
+
         ByteLength = 1;
         break;
 
+    case ACPI_DMT_ASPT:
     case ACPI_DMT_UINT16:
     case ACPI_DMT_DMAR:
+    case ACPI_DMT_ERDT:
     case ACPI_DMT_HEST:
+    case ACPI_DMT_HMAT:
+    case ACPI_DMT_NFIT:
     case ACPI_DMT_PCI_PATH:
+    case ACPI_DMT_PHAT:
+    case ACPI_DMT_RHCT:
+
         ByteLength = 2;
         break;
 
     case ACPI_DMT_UINT24:
+
         ByteLength = 3;
         break;
 
     case ACPI_DMT_UINT32:
+    case ACPI_DMT_AEST_CACHE:
+    case ACPI_DMT_AEST_GIC:
     case ACPI_DMT_NAME4:
     case ACPI_DMT_SIG:
+    case ACPI_DMT_LPIT:
+    case ACPI_DMT_TPM2:
+
         ByteLength = 4;
         break;
 
+    case ACPI_DMT_UINT40:
+
+        ByteLength = 5;
+        break;
+
+    case ACPI_DMT_UINT48:
     case ACPI_DMT_NAME6:
+
         ByteLength = 6;
         break;
 
     case ACPI_DMT_UINT56:
+    case ACPI_DMT_BUF7:
+
         ByteLength = 7;
         break;
 
     case ACPI_DMT_UINT64:
     case ACPI_DMT_NAME8:
+
         ByteLength = 8;
         break;
 
     case ACPI_DMT_STRING:
-        Value = DtGetFieldValue (Field, Info->Name);
 
-        /* TBD: error if Value is NULL? (as below?) */
+        Value = DtGetFieldValue (Field);
+        if (Value)
+        {
+            ByteLength = strlen (Value) + 1;
+        }
+        else
+        {   /* At this point, this is a fatal error */
 
-        ByteLength = ACPI_STRLEN (Value) + 1;
+            sprintf (AslGbl_MsgBuffer, "Expected \"%s\"", Info->Name);
+            DtFatal (ASL_MSG_COMPILER_INTERNAL, NULL, AslGbl_MsgBuffer);
+            return (0);
+        }
+        break;
+
+    case ACPI_DMT_IVRS_UNTERMINATED_STRING:
+
+        Value = DtGetFieldValue (Field);
+        if (Value)
+        {
+            ByteLength = strlen (Value);
+        }
+        else
+        {   /* At this point, this is a fatal error */
+
+            sprintf (AslGbl_MsgBuffer, "Expected \"%s\"", Info->Name);
+            DtFatal (ASL_MSG_COMPILER_INTERNAL, NULL, AslGbl_MsgBuffer);
+            return (0);
+        }
         break;
 
     case ACPI_DMT_GAS:
+
         ByteLength = sizeof (ACPI_GENERIC_ADDRESS);
         break;
 
     case ACPI_DMT_HESTNTFY:
+
         ByteLength = sizeof (ACPI_HEST_NOTIFY);
         break;
 
+    case ACPI_DMT_IORTMEM:
+
+        ByteLength = sizeof (ACPI_IORT_MEMORY_ACCESS);
+        break;
+
     case ACPI_DMT_BUFFER:
-        Value = DtGetFieldValue (Field, Info->Name);
+    case ACPI_DMT_RAW_BUFFER:
+    case ACPI_DMT_PMTT_VENDOR:
+
+        Value = DtGetFieldValue (Field);
         if (Value)
         {
             ByteLength = DtGetBufferLength (Value);
@@ -642,18 +726,77 @@ DtGetFieldLength (
         else
         {   /* At this point, this is a fatal error */
 
-            sprintf (MsgBuffer, "Expected \"%s\"", Info->Name);
-            DtFatal (ASL_MSG_COMPILER_INTERNAL, NULL, MsgBuffer);
+            sprintf (AslGbl_MsgBuffer, "Expected \"%s\"", Info->Name);
+            DtFatal (ASL_MSG_COMPILER_INTERNAL, NULL, AslGbl_MsgBuffer);
+            return (0);
         }
         break;
 
+    case ACPI_DMT_BUF10:
+
+        ByteLength = 10;
+        break;
+
+    case ACPI_DMT_BUF11:
+
+        ByteLength = 11;
+        break;
+
+    case ACPI_DMT_BUF12:
+
+        ByteLength = 12;
+        break;
+
     case ACPI_DMT_BUF16:
+    case ACPI_DMT_UUID:
+
         ByteLength = 16;
         break;
 
-    default:
-        DtFatal (ASL_MSG_COMPILER_INTERNAL, Field, "Invalid table opcode");
+    case ACPI_DMT_BUF18:
+
+        ByteLength = 18;
         break;
+
+    case ACPI_DMT_BUF24:
+
+        ByteLength = 24;
+        break;
+
+    case ACPI_DMT_BUF26:
+
+        ByteLength = 26;
+        break;
+
+    case ACPI_DMT_BUF32:
+
+        ByteLength = 32;
+        break;
+
+    case ACPI_DMT_BUF112:
+
+        ByteLength = 112;
+        break;
+
+    case ACPI_DMT_BUF128:
+
+        ByteLength = 128;
+        break;
+
+    case ACPI_DMT_UNICODE:
+    case ACPI_DMT_WPBT_UNICODE:
+
+        Value = DtGetFieldValue (Field);
+
+        /* TBD: error if Value is NULL? (as below?) */
+
+        ByteLength = (strlen (Value) + 1) * sizeof (UINT16);
+        break;
+
+    default:
+
+        DtFatal (ASL_MSG_COMPILER_INTERNAL, Field, "Invalid table opcode");
+        return (0);
     }
 
     return (ByteLength);
@@ -685,7 +828,7 @@ DtSum (
     UINT8                   *Sum = ReturnValue;
 
 
-    Checksum = AcpiTbChecksum (Subtable->Buffer, Subtable->Length);
+    Checksum = AcpiUtChecksum (Subtable->Buffer, Subtable->Length);
     *Sum = (UINT8) (*Sum + Checksum);
 }
 
@@ -710,7 +853,7 @@ DtSetTableChecksum (
     UINT8                   OldSum;
 
 
-    DtWalkTableTree (Gbl_RootTable, DtSum, NULL, &Checksum);
+    DtWalkTableTree (AslGbl_RootTable, DtSum, NULL, &Checksum);
 
     OldSum = *ChecksumPointer;
     Checksum = (UINT8) (Checksum - OldSum);
@@ -742,7 +885,7 @@ DtSetTableLength (
     DT_SUBTABLE             *ChildTable;
 
 
-    ParentTable = Gbl_RootTable;
+    ParentTable = AslGbl_RootTable;
     ChildTable = NULL;
 
     if (!ParentTable)
@@ -780,7 +923,7 @@ DtSetTableLength (
         {
             ChildTable = ParentTable;
 
-            if (ChildTable == Gbl_RootTable)
+            if (ChildTable == AslGbl_RootTable)
             {
                 break;
             }
@@ -849,7 +992,7 @@ DtWalkTableTree (
         else
         {
             ChildTable = ParentTable;
-            if (ChildTable == Gbl_RootTable)
+            if (ChildTable == AslGbl_RootTable)
             {
                 break;
             }
@@ -863,41 +1006,3 @@ DtWalkTableTree (
         }
     }
 }
-
-
-/******************************************************************************
- *
- * FUNCTION:    DtFreeFieldList
- *
- * PARAMETERS:  None
- *
- * RETURN:      None
- *
- * DESCRIPTION: Free the field list
- *
- *****************************************************************************/
-
-void
-DtFreeFieldList (
-    void)
-{
-    DT_FIELD                *Field = Gbl_FieldList;
-    DT_FIELD                *NextField;
-
-
-    /* Walk and free entire field list */
-
-    while (Field)
-    {
-        NextField = Field->Next; /* Save link */
-
-        if (!(Field->Flags & DT_FIELD_NOT_ALLOCATED))
-        {
-            ACPI_FREE (Field->Name);
-            ACPI_FREE (Field->Value);
-        }
-
-        ACPI_FREE (Field);
-        Field = NextField;
-    }
-}
