@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,11 +25,11 @@
 #include <limits.h>
 
 typedef struct {
-	char *dmi_field;
-	char *label;
+	const char *dmi_field;
+	const char *label;
 } fwts_bios_info;
 
-static fwts_bios_info bios_info[] = {
+static const fwts_bios_info bios_info[] = {
 	{ "bios_vendor",	"BIOS Vendor" },
 	{ "bios_version", 	"BIOS Version" },
 	{ "bios_date",		"BIOS Release Date" },
@@ -43,7 +43,7 @@ static fwts_bios_info bios_info[] = {
 	{ "chassis_version",	"Chassis Version" },
 	{ "chassis_asset_tag",	"Chassic Asset Tag" },
 	{ "product_name",	"Product Name" },
-	{ "product_serial"	"Product Serial #" },
+	{ "product_serial",	"Product Serial #" },
 	{ "product_uuid",	"Product UUID " },
 	{ "product_version",	"Product Version" },
 	{ "sys_vendor",		"System Vendor" },
@@ -55,12 +55,13 @@ static int bios_info_test1(fwts_framework *fw)
 	char path[PATH_MAX];
 	int i;
 
-	for (i=0; bios_info[i].dmi_field != NULL; i++) {
+	for (i = 0; bios_info[i].dmi_field != NULL; i++) {
 		char *data;
-		snprintf(path, sizeof(path), "/sys/class/dmi/id/%s", 
+		snprintf(path, sizeof(path), "/sys/class/dmi/id/%s",
 			bios_info[i].dmi_field);
 		if ((data = fwts_get(path)) != NULL) {
-			fwts_log_info_verbatum(fw, "%-18.18s: %s",
+			fwts_chop_newline(data);
+			fwts_log_info_verbatim(fw, "%-18.18s: %s",
 				bios_info[i].label, data);
 			free(data);
 		}
@@ -81,6 +82,6 @@ static fwts_framework_ops bios_info_ops = {
 	.minor_tests = bios_info_tests
 };
 
-FWTS_REGISTER(bios_info, &bios_info_ops, FWTS_TEST_FIRST, FWTS_BATCH);
+FWTS_REGISTER("bios_info", &bios_info_ops, FWTS_TEST_FIRST, FWTS_FLAG_BATCH)
 
 #endif
