@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2023 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,7 +35,7 @@
 #include <limits.h>
 #include <fcntl.h>
 
-#define DMI_VERSION			(0x0370)
+#define DMI_VERSION			(0x0380)
 #define VERSION_MAJOR(v)		((v) >> 8)
 #define VERSION_MINOR(v)		((v) & 0xff)
 
@@ -56,6 +56,7 @@
 #define DMI_RESERVED_VALUE_USED		"DMIReservedValueUsed"
 #define DMI_RESERVED_BIT_USED		"DMIReservedBitUsed"
 #define DMI_RESERVED_OFFSET_NONZERO	"DMIReservedOffsetNonZero"
+#define DMI_BAD_STRING			"DMIBadString"
 
 #define GET_UINT8(x) (uint8_t)(*(const uint8_t *)(x))
 #define GET_UINT16(x) (uint16_t)(*(const uint16_t *)(x))
@@ -290,6 +291,24 @@ static const fwts_dmi_used_by_kernel dmi_used_by_kernel_table[] = {
 
 static const fwts_dmi_type_length type_info[] = {
 	/* type, max_version, min_version, length */
+	{ 1, 0x201, 0x200, 0x8 },
+	{ 1, 0x204, 0x201, 0x19 },
+	{ 1, MAX_VERSION, 0x204, 0x1b },
+	{ 4, 0x201, 0x200, 0x1a },
+	{ 4, 0x203, 0x201, 0x20 },
+	{ 4, 0x205, 0x203, 0x23 },
+	{ 4, 0x206, 0x205, 0x28 },
+	{ 4, 0x300, 0x206, 0x2a },
+	{ 4, 0x306, 0x300, 0x30 },
+	{ 4, 0x308, 0x306, 0x32 },
+	{ 4, MAX_VERSION, 0x308, 0x33 },
+	{ 7, 0x201, 0x200, 0x0f },
+	{ 7, 0x301, 0x201, 0x13 },
+	{ 7, MAX_VERSION, 0x301, 0x1b },
+	{ 8, MAX_VERSION, 0x200, 0x9 },
+	{ 11, MAX_VERSION, 0x200, 0x5 },
+	{ 12, MAX_VERSION, 0x200, 0x5 },
+	{ 13, MAX_VERSION, 0x200, 0x16 },
 	{ 15, 0x201, 0x200, 0x14 },
 	{ 16, 0x207, 0x201, 0xf },
 	{ 16, MAX_VERSION, 0x207, 0x17 },
@@ -301,10 +320,24 @@ static const fwts_dmi_type_length type_info[] = {
 	{ 17, 0x303, 0x302, 0x54 },
 	{ 17, 0x307, 0x303, 0x5c },
 	{ 17, MAX_VERSION, 0x307, 0x64 },
+	{ 18, MAX_VERSION, 0x201, 0x17 },
 	{ 19, 0x207, 0x201, 0xf },
 	{ 19, MAX_VERSION, 0x207, 0x1f },
 	{ 20, 0x207, 0x201, 0x13 },
 	{ 20, MAX_VERSION, 0x207, 0x23 },
+	{ 21, MAX_VERSION, 0x201, 0x7 },
+	{ 22, MAX_VERSION, 0x201, 0x1a },
+	{ 23, MAX_VERSION, 0x201, 0xd },
+	{ 24, MAX_VERSION, 0x201, 0x5 },
+	{ 25, MAX_VERSION, 0x201, 0x9 },
+	{ 30, MAX_VERSION, 0x201, 0x6 },
+	{ 33, MAX_VERSION, 0x201, 0x1f },
+	{ 34, MAX_VERSION, 0x201, 0xb },
+	{ 35, MAX_VERSION, 0x201, 0xb },
+	{ 36, MAX_VERSION, 0x201, 0x10 },
+	{ 41, MAX_VERSION, 0x201, 0xb },
+	{ 43, MAX_VERSION, 0x201, 0x1f },
+	{ 46, MAX_VERSION, 0x201, 0x9 },
 	{ 0, 0, 0, 0 } /* terminator */
 };
 
@@ -345,6 +378,10 @@ static int dmi_load_file_variable_size(
 		return FWTS_ERROR;
 
 	for (p = buf, total = count = 0; ; p += count) {
+		if (total >= sz) {
+			close(fd);
+			return FWTS_ERROR;
+		}
 		count = read(fd, p, sz - total);
 		if (count == -1) {
 			close(fd);
@@ -389,7 +426,8 @@ static void* dmi_table_smbios(fwts_framework *fw, fwts_smbios_entry *entry)
 	}
 
 #ifdef FWTS_ARCH_AARCH64
-	if (fwts_kernel_config_set("CONFIG_STRICT_DEVMEM")) {
+	if (!fwts_kernel_config_exist() ||
+			fwts_kernel_config_set("CONFIG_STRICT_DEVMEM")) {
 		fwts_warning(fw, "Skipping scanning SMBIOS table in memory for arm64 systems");
 		return NULL;
 	}
@@ -444,7 +482,8 @@ static void* dmi_table_smbios30(fwts_framework *fw, fwts_smbios30_entry *entry)
 	}
 
 #ifdef FWTS_ARCH_AARCH64
-	if (fwts_kernel_config_set("CONFIG_STRICT_DEVMEM")) {
+	if (!fwts_kernel_config_exist() ||
+			fwts_kernel_config_set("CONFIG_STRICT_DEVMEM")) {
 		fwts_warning(fw, "Skipping scanning SMBIOS3 table in memory for arm64 systems");
 		return NULL;
 	}
@@ -1400,6 +1439,67 @@ static void dmicheck_entry(fwts_framework *fw,
 			dmi_min_max_uint8_check(fw, table, addr, "Processor Type", hdr, 0x5, 0x1, 0x6);
 			dmi_min_max_uint8_check(fw, table, addr, "Processor Family", hdr, 0x6, 0x1, 0xfe);
 			dmi_str_check(fw, table, addr, "Processor Manufacturer", hdr, 0x7);
+#ifdef FWTS_ARCH_AARCH64
+			/* Check for Processor ID */
+			if (hdr->length >= 0x28)
+			{
+				/* Get "Processor Characteristics" and check 9th bit if Arm64 SoC ID is supported */
+				uint32_t processor_ID_first_DWORD  = GET_UINT32(data + 0x8);  /* Getting 0x8, 0x9 ,0xA, 0xB */
+				uint32_t processor_ID_second_DWORD = GET_UINT32(data + 0xC);  /* Getting 0xC, 0xD ,0xE, 0xF */
+
+				if(((GET_UINT16(data + 0x26) >> 8) & 0x02) == 0x02)
+				{
+					/* Arm64 SoC ID is Supported. The second DWORD should be non-zero */
+					if (processor_ID_second_DWORD == 0)
+					{
+						fwts_failed(fw, LOG_LEVEL_HIGH, DMI_VALUE_OUT_OF_RANGE,
+							"According to the configured \"Processor Characteristics\", Arm64 SoC ID is supported "
+							"however, the SoC revision value is 0x%8.8" PRIx32 " . Non-zero expected.",
+							processor_ID_second_DWORD );
+
+						fwts_advice(fw,
+							"Arm64 SoC ID support is configured to true in \"Processor Characteristics\" "
+							"however, the SiP SoC revision value is 0. "
+							"It is possible that the system is not correctly configured for "
+							"Arm64 SoC ID and still has the old configuration of "
+							"MIDR_EL1 register. If Arm64 SoC ID is indeed not supported "
+							"then reset the bit in \"Processor Characteristics\".");
+					}
+				}
+				else
+				{
+					/*
+					 * Arm64 SoC ID is NOT Supported
+					 * First DWORD should have contents of MIDR_EL1
+					 * Second DWORD is reserved and must be 0
+					 */
+					if (processor_ID_second_DWORD != 0 )
+					{
+						fwts_failed(fw, LOG_LEVEL_HIGH, DMI_VALUE_OUT_OF_RANGE,
+							"According to the configured \"Processor Characteristics\" Arm64 SoC ID is NOT supported "
+							"however, the DWORD in offset is 0Ch-0Fh is non-zero: 0x%8.8" PRIx32 " ",
+							processor_ID_second_DWORD );
+
+						fwts_advice(fw,
+							"Arm64 SoC ID support is configured to false in \"Processor Characteristics\" "
+							"however, the DWORD in offset is 0Ch-0Fh is non-zero. Configure the "
+							"\"Processor Characteristics\" "
+							"correctly if Arm64 SoC ID is indeed supported in the system.");
+					}
+					if ( processor_ID_first_DWORD == 0 )
+					{
+						fwts_failed(fw, LOG_LEVEL_HIGH, DMI_VALUE_OUT_OF_RANGE,
+							"According to the configured \"Processor Characteristics\" Arm64 SoC ID is NOT supported "
+							"however, the DWORD in offset is 08h-0bh is zero: 0x%8.8" PRIx16 " This should contain MIDR_EL1 value",
+							processor_ID_first_DWORD );
+
+						fwts_advice(fw,
+							"Arm64 SoC ID support is configured to false in \"Processor Characteristics\" "
+							"however, the DWORD in offset is 08h-0bh is zero. This must contain MIDR_EL1");
+					}
+				}
+			}
+#endif
 			dmi_str_check(fw, table, addr, "Processor Version", hdr, 0x10);
 			if (((GET_UINT8(data + 0x18) & 0x07) == 0x5) || ((GET_UINT8(data + 0x18) & 0x07) == 0x6))
 				fwts_failed(fw, LOG_LEVEL_HIGH, DMI_VALUE_OUT_OF_RANGE,
@@ -1411,7 +1511,8 @@ static void dmicheck_entry(fwts_framework *fw,
 					table, addr, "Status", 0x18);
 			dmi_reserved_bits_check(fw, table, addr, "Status", hdr, sizeof(uint8_t), 0x18, 3, 5);
 			dmi_reserved_bits_check(fw, table, addr, "Status", hdr, sizeof(uint8_t), 0x18, 7, 7);
-			dmi_min_max_uint8_check(fw, table, addr, "Processor Upgrade", hdr, 0x19, 0x1, 0x50);
+			fwts_dmi_value_range t4_ranges[] = {{0x1, 0x57}, {0xff, 0xff}};
+			dmi_ranges_uint8_check(fw, table, addr, "Processor Upgrade", hdr, 0x19, t4_ranges);
 			if (hdr->length < 0x23)
 				break;
 			dmi_str_check(fw, table, addr, "Serial Number", hdr, 0x20);
@@ -1432,6 +1533,16 @@ static void dmicheck_entry(fwts_framework *fw,
 			if (hdr->length < 0x32)
 				break;
 			dmi_min_max_uint16_check(fw, table, addr, "Thread Enabled", hdr, 0x30, 0, 0xfffe);
+			if (hdr->length < 0x33)
+				break;
+			dmi_str_check(fw, table, addr, "Socket Type", hdr, 0x32);
+			if (GET_UINT8(data + 0x19) == 0xFF && ((char)data[0x32] == '\0'))
+				fwts_failed(fw, LOG_LEVEL_HIGH,
+					DMI_BAD_STRING,
+					"Must be non-null if 'Processor Upgrade' at offset 19h is FFh "
+					"while accessing entry '%s' @ 0x%8.8" PRIx32
+					"on field '%s', offset 0x%2.2" PRIx8,
+					table, addr, "Socket Type", 0x32);
 			break;
 
 		case 5: /* 7.6 (Type 5 is obsolete) */
@@ -1655,7 +1766,7 @@ static void dmicheck_entry(fwts_framework *fw,
 				break;
 			if (hdr->length < 0x15)
 				break;
-			dmi_min_max_uint8_check(fw, table, addr, "Form Factor", hdr, 0xe, 0x1, 0x10);
+			dmi_min_max_uint8_check(fw, table, addr, "Form Factor", hdr, 0xe, 0x1, 0x11);
 			dmi_str_check(fw, table, addr, "Locator", hdr, 0x10);
 			dmi_str_check(fw, table, addr, "Bank Locator", hdr, 0x11);
 			fwts_dmi_value_range t17_ranges[] = {{0x1, 0x14}, {0x18, 0x24}};
@@ -1675,10 +1786,18 @@ static void dmicheck_entry(fwts_framework *fw,
 			dmi_reserved_bits_check(fw, table, addr, "Extended Size", hdr, sizeof(uint32_t), 0x1c, 31, 31);
 			if (hdr->length < 0x4c)
 				break;
-			dmi_min_max_uint8_check(fw, table, addr, "Memory Technology", hdr, 0x28, 0x1, 0x7);
+			dmi_min_max_uint8_check(fw, table, addr, "Memory Technology", hdr, 0x28, 0x1, 0x8);
 			dmi_reserved_bits_check(fw, table, addr, "Memory Operating Mode Cap", hdr, sizeof(uint16_t), 0x29, 0, 0);
 			dmi_reserved_bits_check(fw, table, addr, "Memory Operating Mode Cap", hdr, sizeof(uint16_t), 0x29, 6, 15);
-			dmi_str_check(fw, table, addr, "Firmware Version", hdr, 0x2b);
+			switch(hdr->data[0x28]) {
+			case 0x04:
+			case 0x05:
+			case 0x06:
+				dmi_str_check(fw, table, addr, "Firmware Version", hdr, 0x2b);
+				break;
+			default:
+				break;
+			}
 			if (hdr->length < 0x54)
 				break;
 			dmi_reserved_bits_check(fw, table, addr, "Extended Speed", hdr, sizeof(uint32_t), 0x54, 31, 31);
