@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2023 Canonical
+ * Copyright (C) 2010-2025 Canonical
  * Some of this work - Copyright (C) 2016-2021 IBM
  *
  * This program is free software; you can redistribute it and/or
@@ -127,7 +127,14 @@
 #define FWTS_USE_DEVMEM 1
 #endif
 
-/* verision 3-tuple into integer */
+#if defined(__loongarch__)
+#define FWTS_ARCH_LOONGARCH	1
+#define FWTS_HAS_ACPI	1
+#define FWTS_HAS_UEFI	1
+#define FWTS_USE_DEVMEM	1
+#endif
+
+/* version 3-tuple into integer */
 #define _VER_(major, minor, patchlevel)                 \
 	((major * 10000) + (minor * 100) + patchlevel)
 
