@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2023 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,7 +56,8 @@
 #define FWTS_FACP_FLAG_FORCE_APIC_PHYSICAL_DESTINATION_MODE	(0x00080000)
 #define FWTS_FACP_FLAG_HW_REDUCED_ACPI				(0x00100000)
 #define FWTS_FACP_FLAG_LOW_POWER_S0_IDLE_CAPABLE		(0x00200000)
-#define FWTS_FACP_FLAG_RESERVED_MASK				(0xffc00000)
+#define FWTS_FACP_FLAG_PERSISTENT_CPU_CACHES			(0x00c00000)
+#define FWTS_FACP_FLAG_RESERVED_MASK				(0xff000000)
 
 #define FWTS_FACP_IAPC_BOOT_ARCH_LEGACY_DEVICES		(0x0001)
 #define FWTS_FACP_IAPC_BOOT_ARCH_8042			(0x0002)
@@ -85,10 +86,12 @@
 #define FWTS_GAS_ADDR_SPACE_ID_GPIO		(0x08)
 #define FWTS_GAS_ADDR_SPACE_ID_GSB		(0x09)
 #define FWTS_GAS_ADDR_SPACE_ID_PCC		(0x0a)
+#define FWTS_GAS_ADDR_SPACE_ID_PRM		(0x0b)
 #define FWTS_GAS_ADDR_SPACE_ID_FFH		(0x7f)
 
-#define FWTS_ACPI_VERSION_NOW	(FWTS_ACPI_VERSION_65)
-#define FWTS_ACPI_VERSION_NEXT	(0x660)
+#define FWTS_ACPI_VERSION_NOW	(FWTS_ACPI_VERSION_66)
+#define FWTS_ACPI_VERSION_NEXT	(0x670)
+#define FWTS_ACPI_VERSION_66	(0x660)
 #define FWTS_ACPI_VERSION_65	(0x650)
 #define FWTS_ACPI_VERSION_64 	(0x640)
 #define FWTS_ACPI_VERSION_63 	(0x630)
@@ -426,6 +429,16 @@ typedef struct {
 	uint32_t	clock_domain;
 } __attribute__ ((packed)) fwts_acpi_table_gicc_affinity;
 
+typedef struct {
+	uint8_t		type;
+	uint8_t		length;
+	uint16_t	reserved;
+	uint32_t	proximity_domain;
+	uint32_t	acpi_processor_uid;
+	uint32_t	flags;
+	uint32_t	clock_domain;
+} __attribute__ ((packed)) fwts_acpi_table_rintc_affinity;
+
 typedef struct {
 	uint8_t		type;
 	uint8_t		length;
@@ -445,6 +458,17 @@ typedef struct {
 	uint32_t	reserved2;
 } __attribute__ ((packed)) fwts_acpi_table_initiator_affinity;
 
+typedef struct {
+	uint8_t		type;
+	uint8_t		length;
+	uint8_t		reserved1;
+	uint8_t		device_handle_type;
+	uint32_t	proximity_domain;
+	uint8_t		device_handle[16];
+	uint32_t	flags;
+	uint32_t	reserved2;
+} __attribute__ ((packed)) fwts_acpi_table_port_affinity;
+
 /*
  * ACPI HPET (HPET IA-PC High Precision Event Timer Table),
  *    3.2.4 The ACPI 2.0 HPET Description Table (HPET)
@@ -531,6 +555,10 @@ typedef enum {
 	FWTS_MADT_MSI_PIC,
 	FWTS_MADT_BIO_PIC,
 	FWTS_MADT_LPC_PIC,
+	FWTS_MADT_RINTC,
+	FWTS_MADT_IMSIC,
+	FWTS_MADT_APLIC,
+	FWTS_MADT_PLIC,
 	FWTS_MADT_RESERVED, /* does not have defined structure */
 	FWTS_MADT_OEM /* does not have defined structure */
 } fwts_acpi_madt_type;
@@ -669,7 +697,13 @@ typedef struct {
 /* New in ACPI 5.1, GICR structure, 5.2.12.17 */
 /* Type 14, FWTS_MADT_GIC_R_REDISTRIBUTOR */
 typedef struct {
-	uint16_t	reserved;
+	union {
+		uint16_t	reserved;
+		struct {
+			uint8_t	flags;
+			uint8_t	reserved;
+		} __attribute__ ((packed)) gicr;
+	};
 	uint64_t	discovery_range_base_address;
 	uint32_t	discovery_range_length;
 } __attribute__ ((packed)) fwts_acpi_madt_gicr;
@@ -677,7 +711,13 @@ typedef struct {
 /* New in ACPI 6.0, GIC ITS structure, 5.2.12.18 */
 /* Type 15, FWTS_MADT_GIC_ITS */
 typedef struct {
-	uint16_t	reserved;
+	union {
+		uint16_t	reserved;
+		struct {
+			uint8_t	flags;
+			uint8_t	reserved;
+		} __attribute__ ((packed)) gic_its;
+	};
 	uint32_t	its_id;
 	uint64_t	physical_base_address;
 	uint32_t	reserved2;
@@ -689,6 +729,7 @@ typedef struct {
 	uint16_t	mail_box_version;
 	uint32_t	reserved;
 	uint64_t	mail_box_address;
+	uint64_t	reset_vector;	/* Present (ACPI 6.6) */
 } __attribute__ ((packed)) fwts_acpi_madt_mp_wakeup;
 
 /* New in ACPI 6.5 Core Programmable Interrupt Controller (CORE PIC) Structure, 5.2.12.20 */
@@ -700,6 +741,57 @@ typedef struct {
 	uint32_t	flag;
 } __attribute__ ((packed)) fwts_acpi_madt_core_pic;
 
+/* Type 24, FWTS_MADT_RINTC */
+typedef struct {
+	uint8_t version;
+	uint8_t reserved;
+	uint32_t flags;
+	uint64_t hart_id;
+	uint32_t uid;
+	uint32_t ext_intc_id;
+	uint64_t imsic_addr;
+	uint32_t imsic_size;
+} __attribute__ ((packed)) fwts_acpi_madt_rintc;
+
+/* Type 25, FWTS_MADT_IMSIC */
+typedef struct {
+	uint8_t version;
+	uint8_t reserved;
+	uint32_t flags;
+	uint16_t num_ids;
+	uint16_t num_guest_ids;
+	uint8_t guest_index_bits;
+	uint8_t hart_index_bits;
+	uint8_t group_index_bits;
+	uint8_t group_index_shift;
+} __attribute__ ((packed)) fwts_acpi_madt_imsic;
+
+/* Type 26, FWTS_MADT_APLIC */
+typedef struct {
+	uint8_t version;
+	uint8_t id;
+	uint32_t flags;
+	uint8_t hw_id[8];
+	uint16_t num_idcs;
+	uint16_t num_sources;
+	uint32_t gsi_base;
+	uint64_t addr;
+	uint32_t size;
+} __attribute__ ((packed)) fwts_acpi_madt_aplic;
+
+/* Type 27, FWTS_MADT_PLIC */
+typedef struct {
+	uint8_t version;
+	uint8_t id;
+	uint8_t hw_id[8];
+	uint16_t num_irqs;
+	uint16_t max_prio;
+	uint32_t flags;
+	uint32_t size;
+	uint64_t base_addr;
+	uint32_t gsi_base;
+} __attribute__ ((packed)) fwts_acpi_madt_plic;
+
 /*
  * ACPI TCPA (Trusted Computing Platform Alliance Capabilities Table)
  *   http://www.kuro5hin.org/story/2002/10/27/16622/530,
@@ -1022,6 +1114,25 @@ typedef struct {
 	uint64_t	fbpt_addr;
 } __attribute__ ((packed)) fwts_acpi_table_fpdt_basic_boot_perf_ptr;
 
+/*
+ * ACPI Microcontroller Boot Performance Table Pointer Record, 5.2.24.6
+ */
+typedef struct {
+	fwts_acpi_table_fpdt_header	fpdt;
+	uint32_t	reserved;
+	uint64_t	mbpt_addr;
+} __attribute__ ((packed)) fwts_acpi_table_fpdt_micro_boot_perf_ptr;
+
+/*
+ * ACPI Timestamp Delta Record, 5.2.24.7
+ */
+typedef struct {
+	fwts_acpi_table_fpdt_header	fpdt;
+	uint32_t	reserved;
+	uint64_t	timestamp_domain_id;
+	uint64_t	timestamp_delta;
+} __attribute__ ((packed)) fwts_acpi_table_fpdt_timestamp_delta;
+
 /*
  * ACPI GTDT (Generic Timer Description Table), 5.2.24
  */
@@ -2260,19 +2371,81 @@ typedef struct {
 
 /*
  *  ACPI ASPT
- *	determined by reverse engineering
+ *	https://www.amd.com/content/dam/amd/en/documents/epyc-technical-docs/specifications/58028_1_00-PUB.pdf
  */
+typedef struct {
+	uint16_t	type;
+	uint16_t	length;
+} __attribute__ ((packed)) fwts_aspt_sub_header;
+
+typedef struct {
+	fwts_aspt_sub_header header;
+	uint32_t	reserved;
+	union {
+		struct{
+			uint64_t	feature_reg_addr;
+			uint64_t	interrupt_enable_reg_addr;
+			uint64_t	interrupt_stable_reg_addr;
+		} __attribute__ ((packed)) v1;
+		struct {
+			uint32_t	feature_reg_offset;
+			uint32_t	interrupt_enable_reg_offset;
+			uint32_t	interrupt_stable_reg_offset;
+		} __attribute__ ((packed)) v2;
+	};
+} __attribute__ ((packed)) fwts_acpi_table_asp_global;
+
+typedef struct {
+	fwts_aspt_sub_header header;
+	uint8_t		mailbox_interrupt_id;
+	uint8_t		reserved[3];
+	union {
+		struct{
+			uint64_t	cmdresp_reg_addr;
+			uint64_t	cmdbufaddr_lo_reg_addr;
+			uint64_t	cmdbufaddr_hi_reg_addr;
+		} __attribute__ ((packed)) v1;
+		struct {
+			uint32_t	cmdresp_reg_offset;
+			uint32_t	cmdbufaddr_lo_reg_offset;
+			uint32_t	cmdbufaddr_hi_reg_offset;
+		} __attribute__ ((packed)) v2;
+	};
+} __attribute__ ((packed)) fwts_acpi_table_sev_mailbox;
+
+typedef struct {
+	fwts_aspt_sub_header header;
+	uint32_t	reserved;
+	union {
+		struct{
+			uint64_t	cmdresp_reg_addr;
+			uint8_t		reserved1[16];
+		} __attribute__ ((packed)) v1;
+		struct {
+			uint32_t	cmdresp_reg_offset;
+			uint8_t		reserved1[8];
+		} __attribute__ ((packed)) v2;
+	};
+} __attribute__ ((packed)) fwts_acpi_table_acpi_mailbox;
+
 typedef struct {
 	fwts_acpi_table_header  header;
-	uint32_t	sptt_addr_start;
-	uint32_t	sptt_addr_end;
-	uint32_t	amrt_addr_start;
-	uint32_t	amrt_addr_end;
+	union {
+		struct{
+			uint32_t	asp_reg_count;
+		} __attribute__ ((packed)) v1;
+		struct {
+			uint64_t	asp_reg_base_addr;
+			uint32_t	asp_reg_apace_pages;
+			uint32_t	asp_reg_count;
+		} __attribute__ ((packed)) v2;
+	};
+	uint8_t		asp_reg_structure[0];
 } __attribute__ ((packed)) fwts_acpi_table_aspt;
 
 /*
  *  NHLT Table
- *   see https://01.org/sites/default/files/595976_intel_sst_nhlt.pdf
+ *   see 5.2.27 Non HDAudio Link Table (NHLT)
 */
 typedef struct {
 	fwts_acpi_table_header  header;
@@ -2280,7 +2453,7 @@ typedef struct {
 } __attribute__ ((packed)) fwts_acpi_table_nhlt;
 
 typedef struct  {
-	uint32_t	ep_descriptor_len;
+	uint32_t	len;
 	uint8_t		link_type;
 	uint8_t		instance_id;
 	uint16_t	vendor_id;
@@ -2295,29 +2468,40 @@ typedef struct  {
 typedef struct {
 	uint32_t capabilities_size;
 	uint8_t capabilities[0];
-} __attribute__ ((packed)) fwts_acpi_table_nhlt_specific_config;
+} __attribute__ ((packed)) fwts_acpi_table_nhlt_config_space_common;
 
 typedef struct {
-	uint16_t wformattag;
-	uint16_t nchannels;
-	uint32_t nsamplespersec;
-	uint32_t navgbytespersec;
-	uint16_t nblockalign;
-	uint16_t wbitspersample;
-	uint16_t cbsize;
-	uint16_t wvalidbitspersample;
-	uint32_t dwchannelmask;
+	uint8_t		id[16];
+	uint8_t		instance_id;
+	uint8_t		port_id;
+} __attribute__ ((packed)) fwts_acpi_table_nhlt_device_info;
+
+typedef struct {
+	uint8_t		devices_count;
+	fwts_acpi_table_nhlt_device_info devices[0];
+} __attribute__ ((packed)) fwts_acpi_table_nhlt_devices_info;
+
+typedef struct {
+	uint16_t format_tag;
+	uint16_t channels;
+	uint32_t samples_per_sec;
+	uint32_t avg_bytes_per_sec;
+	uint16_t block_align;
+	uint16_t bits_per_sample;
+	uint16_t size;
+	uint16_t valid_bits_per_sample;
+	uint32_t channel_mask;
 	uint8_t	subformat[16];
-} __attribute__ ((packed))  fwts_acpi_table_nhlt_waveformatextensible;
+} __attribute__ ((packed)) fwts_acpi_table_nhlt_wave_format_ext;
 
 typedef struct {
-	fwts_acpi_table_nhlt_waveformatextensible format;
-	fwts_acpi_table_nhlt_specific_config formatconfiguration;
+	fwts_acpi_table_nhlt_wave_format_ext format;
+	fwts_acpi_table_nhlt_config_space_common config;
 } __attribute__ ((packed))  fwts_acpi_table_nhlt_format_configuration;
 
 typedef struct {
-	uint8_t formatscount;
-	fwts_acpi_table_nhlt_format_configuration formatsconfiguration[0];
+	uint8_t formats_count;
+	fwts_acpi_table_nhlt_format_configuration formats[0];
 } __attribute__ ((packed))  fwts_acpi_table_nhlt_formats_configuration;
 
 /*
@@ -2362,7 +2546,8 @@ typedef enum {
 	FWTS_CEDT_TYPE_CFMWS	= 1,
 	FWTS_CEDT_TYPE_CXIMS	= 2,
 	FWTS_CEDT_TYPE_RDPAS	= 3,
-	FWTS_CEDT_TYPE_RESERVED	= 4,
+	FWTS_CEDT_TYPE_CSDS	= 4,
+	FWTS_CEDT_TYPE_RESERVED	= 5,
 } fwts_acpi_cedt_type;
 
 typedef struct {
@@ -2410,6 +2595,12 @@ typedef struct {
 	uint64_t	base_addr;
 } __attribute__ ((packed)) fwts_acpi_table_cedt_rdpas;
 
+typedef struct {
+	fwts_acpi_table_cedt_header	header;
+	uint16_t	system_cap;
+	uint16_t	reserved;
+} __attribute__ ((packed)) fwts_acpi_table_cedt_csds;
+
 /*
  * ACPI AEST (Arm Error Source Table)
  * https://developer.arm.com/documentation/den0085/latest/
@@ -2734,4 +2925,206 @@ typedef struct {
 	uint64_t		reserved;
 } __attribute__ ((packed)) fwts_acpi_table_ibft;
 
+/*
+ * Virtual I/O Translation (VIOT) Table
+ * ACPI6.5 5.2.32
+ */
+typedef struct
+{
+
+	uint8_t		type;
+	uint8_t		reserved;
+	uint16_t	length;
+} __attribute__ ((packed)) fwts_acpi_viot_node_hdr;
+
+typedef struct
+{
+	fwts_acpi_viot_node_hdr hdr;
+	uint32_t	endpoint_start;
+	uint16_t	pci_seg_start;
+	uint16_t	pci_seg_end;
+	uint16_t	pci_bdf_start;
+	uint16_t	pci_bdf_end;
+	uint16_t	output_node;
+	uint8_t		reserved[6];
+} __attribute__ ((packed)) fwts_acpi_viot_pci_rng_node;
+
+typedef struct
+{
+	fwts_acpi_viot_node_hdr hdr;
+	uint32_t	endpoint_id;
+	uint64_t	base_addr;
+	uint16_t	output_node;
+	uint8_t		reserved[6];
+} __attribute__ ((packed)) fwts_acpi_viot_mmio_ep_node;
+
+typedef struct
+{
+	fwts_acpi_viot_node_hdr hdr;
+	uint16_t	pci_seg;
+	uint16_t	pci_bdf_num;
+	uint64_t	reserved;
+} __attribute__ ((packed)) fwts_acpi_viot_pci_iommu_node;
+
+typedef struct
+{
+	fwts_acpi_viot_node_hdr hdr;
+	uint32_t	reserved;
+	uint64_t	base_addr;
+} __attribute__ ((packed)) fwts_acpi_viot_mmio_iommu_node;
+
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint16_t		node_count;
+	uint16_t		node_offset;
+	uint64_t		reserved;
+} __attribute__ ((packed)) fwts_acpi_table_viot;
+
+/*
+ * Miscellaneous GUIDed Table Entries
+ * ACPI6.5 5.2.33
+ */
+typedef struct {
+	uint8_t		entry_guid[16];
+	uint32_t	entry_len;
+	uint32_t	revision;
+	uint32_t	producer_id;
+	uint8_t		data[0];
+} __attribute__ ((packed)) fwts_acpi_misc_guided_entry;
+
+typedef struct {
+	fwts_acpi_table_header		header;
+	fwts_acpi_misc_guided_entry 	entry[0];
+} __attribute__ ((packed)) fwts_acpi_table_misc;
+
+/*
+ * CC Event Log ACPI Table
+ * ACPI6.5 5.2.34
+ */
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint8_t			cc_type;
+	uint8_t			cc_subtype;
+	uint16_t		reserved;
+	uint64_t		laml;
+	uint64_t		lasa;
+} __attribute__ ((packed)) fwts_acpi_table_ccel;
+
+/*
+ * Storage Volume Key Location Table
+ * ACPI6.5 5.2.35
+ */
+typedef struct {
+	uint16_t	key_type;
+	uint16_t	key_format;
+	uint32_t	key_size;
+	uint64_t	key_address;
+} __attribute__ ((packed)) fwts_acpi_skvl_key_struct;
+
+typedef struct {
+	fwts_acpi_table_header		header;
+	uint32_t			key_count;
+	fwts_acpi_skvl_key_struct 	key_struct[0];
+} __attribute__ ((packed)) fwts_acpi_table_skvl;
+
+/*
+ * S3 Performance Table
+ * ACPI6.5 5.2.24.8
+ */
+typedef struct {
+	char		signature[4];
+	uint32_t	length;
+} __attribute__ ((packed)) fwts_acpi_table_s3pt_header;
+
+typedef struct {
+	uint16_t	rt_perf_record_type;
+	uint8_t		record_len;
+	uint8_t		revision;
+} __attribute__ ((packed)) fwts_acpi_s3pt_record_hdr;
+
+typedef struct {
+	fwts_acpi_s3pt_record_hdr 	header;
+	uint32_t			resume_count;
+	uint64_t			full_resume;
+	uint64_t			average_resume;
+} __attribute__ ((packed)) fwts_acpi_s3pt_resume;
+
+typedef struct {
+	fwts_acpi_s3pt_record_hdr 	header;
+	uint64_t			suspend_start;
+	uint64_t			suspend_end;
+} __attribute__ ((packed)) fwts_acpi_s3pt_suspend;
+
+/*
+ * ACPI APMT(Arm Performance Monitoring Unit table)
+ * https://developer.arm.com/documentation/den0117/latest/
+ */
+
+typedef struct {
+	uint16_t	length;
+	uint8_t		flags;
+	uint8_t		type;
+	uint32_t	identifier;
+	uint64_t	instance_prim;
+	uint32_t	instance_secondary;
+	uint64_t	base_addr_0;
+	uint64_t	base_addr_1;
+	uint32_t	overflow_intr;
+	uint32_t	reserved1;
+	uint32_t	overflow_intr_flag;	
+	uint32_t	processor_affinity;
+	uint32_t	implementation_id;
+} __attribute__ ((packed)) fwts_acpi_apmt_node;
+
+typedef struct {
+	fwts_acpi_table_header	header;
+	fwts_acpi_apmt_node 	entry[0];
+} __attribute__ ((packed)) fwts_acpi_table_apmt;
+
+/*
+ * ACPI RHCT (RISC-V Hart Capabilities Table), 5.2.37
+ */
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint32_t	flags;
+	uint64_t	time_base_freq;
+	uint32_t	node_count;
+	uint32_t	node_offset;
+} __attribute__ ((packed)) fwts_acpi_table_rhct;
+
+typedef struct {
+	uint16_t	type;
+	uint16_t	length;
+	uint16_t	revision;
+} __attribute__ ((packed)) fwts_acpi_rhct_node_header;
+
+typedef enum {
+	FWTS_RHCT_NODE_ISA_STRING = 0x0,
+	FWTS_RHCT_NODE_CMO,
+	FWTS_RHCT_NODE_MMU,
+	FWTS_RHCT_NODE_HART_INFO = 0xFFFF
+} fwts_acpi_rhct_node_type;
+
+typedef struct {
+	uint16_t	isa_length;
+	char		isa[];
+} __attribute__ ((packed)) fwts_acpi_rhct_node_isa_string;
+
+typedef struct {
+	uint8_t	reserved;
+	uint8_t	cbom_size;
+	uint8_t	cbop_size;
+	uint8_t	cboz_size;
+} __attribute__ ((packed)) fwts_acpi_rhct_node_cmo;
+
+typedef struct {
+	uint8_t	reserved;
+	uint8_t	mmu_type;
+} __attribute__ ((packed)) fwts_acpi_rhct_node_mmu;
+
+typedef struct {
+	uint16_t	num_offsets;
+	uint32_t	uid;
+} __attribute__ ((packed)) fwts_acpi_rhct_node_hart_info;
+
 #endif
