@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2011 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 
 #include "fwts.h"
 
-void fwts_dump_raw_data(char *buffer, const int len, const uint8_t *data, const int where, const int bytes);
-
+void fwts_dump_raw_data(char *buffer, const size_t len, const uint8_t *data, const int where, const size_t bytes);
+void fwts_dump_raw_data_prefix(char *buffer, const size_t len, const uint8_t *data, const char *prefix, const size_t bytes);
+void fwts_hexdump_data_prefix_all(fwts_framework *fw, const uint8_t *data, const char *prefix,	const size_t nbytes);
 #endif
