@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2011 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,6 @@
 
 #include "fwts_list.h"
 
-fwts_list *fwts_format_text(const char *text, const int width);
+fwts_list *fwts_format_text(const char *text, const size_t width);
 
 #endif
