@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2011 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,15 +16,18 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
+
 #ifndef __FWTS_WAKEALARM_H__
 #define __FWTS_WAKEALARM_H__
 
+#include <linux/rtc.h>
+
 #include "fwts_framework.h"
 
-#define WAKEALARM "/sys/class/rtc/rtc0/wakealarm"
-
-int fwts_wakealarm_get_irq_state(void);
-int fwts_wakealarm_test_firing(fwts_framework *fw, const int sleep);
-int fwts_wakealarm_trigger(fwts_framework *fw, const int seconds);
+int fwts_wakealarm_test_firing(fwts_framework *fw, const uint32_t sleep);
+int fwts_wakealarm_trigger(fwts_framework *fw, const uint32_t seconds);
+int fwts_wakealarm_cancel(fwts_framework *fw);
+int fwts_wakealarm_get(fwts_framework *fw, struct rtc_time *rtc_tm);
+int fwts_wakealarm_set(fwts_framework *fw, struct rtc_time *rtc_tm);
 
 #endif
