@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Canonical
+ * Copyright (C) 2010-2025 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,14 +21,14 @@
 
 /*
  *  fwts_checksum()
- *	checksum a uint8_t * buffer 
+ *	checksum a uint8_t * buffer
  */
-uint8_t fwts_checksum(const uint8_t *data, const int length)
+uint8_t fwts_checksum(const uint8_t *data, const size_t length)
 {
-	int 	i;
+	size_t i;
 	uint8_t checksum = 0;
 
-	for (i=0; i < length; i++)
+	for (i = 0; i < length; i++)
 		checksum += *data++;
 
 	return checksum;
